@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2011-2016 Free Software Foundation, Inc.
- * Copyright (C) 2016 Red Hat, Inc.
+ * Copyright (C) 2016-2018 Red Hat, Inc.
  *
  * Author: Nikos Mavrogiannopoulos
  *
@@ -37,6 +37,7 @@
 #include <byteswap.h>
 
 #define GCM_BLOCK_SIZE 16
+#define INC32(block) INCREMENT(4, block + GCM_BLOCK_SIZE - 4)
 
 /* GCM mode */
 
@@ -59,6 +60,8 @@ struct gcm128_context {
 struct aes_gcm_ctx {
 	AES_KEY expanded_key;
 	struct gcm128_context gcm;
+	unsigned finished;
+	unsigned auth_finished;
 };
 
 void gcm_init_v8(u128 Htable[16], const uint64_t Xi[2]);
@@ -98,6 +101,8 @@ aes_gcm_cipher_setkey(void *_ctx, const void *userkey, size_t keysize)
 	struct aes_gcm_ctx *ctx = _ctx;
 	int ret;
 
+	CHECK_AES_KEYSIZE(keysize);
+
 	ret =
 	    aes_v8_set_encrypt_key(userkey, keysize * 8,
 				  ALIGN16(&ctx->expanded_key));
@@ -133,6 +138,8 @@ static int aes_gcm_setiv(void *_ctx, const void *iv, size_t iv_size)
 	aes_v8_encrypt(ctx->gcm.Yi.c, ctx->gcm.EK0.c,
 			ALIGN16(&ctx->expanded_key));
 	ctx->gcm.Yi.c[GCM_BLOCK_SIZE - 1] = 2;
+	ctx->finished = 0;
+	ctx->auth_finished = 0;
 	return 0;
 }
 
@@ -169,7 +176,7 @@ ctr32_encrypt_blocks_inplace(const unsigned char *in, unsigned char *out,
 
 		out += 16;
 		in += 16;
-		INCREMENT(16, ctr);
+		INC32(ctr);
 	}
 }
 
@@ -192,7 +199,7 @@ ctr32_encrypt_blocks(const unsigned char *in, unsigned char *out,
 
 		out += 16;
 		in += 16;
-		INCREMENT(16, ctr);
+		INC32(ctr);
 	}
 }
 
@@ -222,6 +229,9 @@ aes_gcm_encrypt(void *_ctx, const void *src, size_t src_size,
 	int rest = src_size - (exp_blocks);
 	uint32_t counter;
 
+	if (unlikely(ctx->finished))
+		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+
 	if (blocks > 0) {
 		ctr32_encrypt_blocks(src, dst,
 				     blocks,
@@ -233,8 +243,10 @@ aes_gcm_encrypt(void *_ctx, const void *src, size_t src_size,
 		_gnutls_write_uint32(counter, ctx->gcm.Yi.c + 12);
 	}
 
-	if (rest > 0)		/* last incomplete block */
+	if (rest > 0) {	/* last incomplete block */
 		ctr_encrypt_last(ctx, src, dst, exp_blocks, rest);
+		ctx->finished = 1;
+	}
 
 	gcm_ghash(ctx, dst, src_size);
 	ctx->gcm.len.u[1] += src_size;
@@ -252,6 +264,9 @@ aes_gcm_decrypt(void *_ctx, const void *src, size_t src_size,
 	int rest = src_size - (exp_blocks);
 	uint32_t counter;
 
+	if (unlikely(ctx->finished))
+		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+
 	gcm_ghash(ctx, src, src_size);
 	ctx->gcm.len.u[1] += src_size;
 
@@ -266,8 +281,10 @@ aes_gcm_decrypt(void *_ctx, const void *src, size_t src_size,
 		_gnutls_write_uint32(counter, ctx->gcm.Yi.c + 12);
 	}
 
-	if (rest > 0)		/* last incomplete block */
+	if (rest > 0) { /* last incomplete block */
 		ctr_encrypt_last(ctx, src, dst, exp_blocks, rest);
+		ctx->finished = 1;
+	}
 
 	return 0;
 }
@@ -276,9 +293,15 @@ static int aes_gcm_auth(void *_ctx, const void *src, size_t src_size)
 {
 	struct aes_gcm_ctx *ctx = _ctx;
 
+	if (unlikely(ctx->auth_finished))
+		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+
 	gcm_ghash(ctx, src, src_size);
 	ctx->gcm.len.u[0] += src_size;
 
+	if (src_size % GCM_BLOCK_SIZE != 0)
+		ctx->auth_finished = 1;
+
 	return 0;
 }
 
