@@ -31,8 +31,9 @@
  * View first: "The order of encryption and authentication for
  * protecting communications" by Hugo Krawczyk - CRYPTO 2001
  *
- * On update, make sure to update MAX_CIPHER_BLOCK_SIZE and MAX_CIPHER_KEY_SIZE
- * as well. If any ciphers are removed, remove them from the back-end but
+ * On update, make sure to update MAX_CIPHER_BLOCK_SIZE, MAX_CIPHER_IV_SIZE,
+ * and MAX_CIPHER_KEY_SIZE as well.
+ * If any ciphers are removed, remove them from the back-end but
  * keep them in that list to allow backwards compatibility with applications
  * that specify them (they will be a no-op).
  */
@@ -164,6 +165,8 @@ static const cipher_entry_st algorithms[] = {
 	  .explicit_iv = 0,
 	  .xor_nonce = 1,
 	  .cipher_iv = 12,
+	  /* in chacha20 we don't need a rekey after 2^24 messages */
+	  .no_rekey = 1,
 	  .tagsize = 16
 	},
 	{ .name = "CAMELLIA-128-GCM",
@@ -180,6 +183,41 @@ static const cipher_entry_st algorithms[] = {
 	  .explicit_iv = 8,
 	  .cipher_iv = 12,
 	  .tagsize = 16},
+	{ .name = "GOST28147-TC26Z-CFB",
+	  .id = GNUTLS_CIPHER_GOST28147_TC26Z_CFB,
+	  .blocksize = 8,
+	  .keysize = 32,
+	  .type = CIPHER_STREAM,
+	  .implicit_iv = 8,
+	  .cipher_iv = 8},
+	{ .name = "GOST28147-CPA-CFB",
+	  .id = GNUTLS_CIPHER_GOST28147_CPA_CFB,
+	  .blocksize = 8,
+	  .keysize = 32,
+	  .type = CIPHER_STREAM,
+	  .implicit_iv = 8,
+	  .cipher_iv = 8},
+	{ .name = "GOST28147-CPB-CFB",
+	  .id = GNUTLS_CIPHER_GOST28147_CPB_CFB,
+	  .blocksize = 8,
+	  .keysize = 32,
+	  .type = CIPHER_STREAM,
+	  .implicit_iv = 8,
+	  .cipher_iv = 8},
+	{ .name = "GOST28147-CPC-CFB",
+	  .id = GNUTLS_CIPHER_GOST28147_CPC_CFB,
+	  .blocksize = 8,
+	  .keysize = 32,
+	  .type = CIPHER_STREAM,
+	  .implicit_iv = 8,
+	  .cipher_iv = 8},
+	{ .name = "GOST28147-CPD-CFB",
+	  .id = GNUTLS_CIPHER_GOST28147_CPD_CFB,
+	  .blocksize = 8,
+	  .keysize = 32,
+	  .type = CIPHER_STREAM,
+	  .implicit_iv = 8,
+	  .cipher_iv = 8},
 	{ .name = "3DES-CBC", 
 	  .id = GNUTLS_CIPHER_3DES_CBC,
 	  .blocksize = 8,
