@@ -24,7 +24,7 @@
 #include <algorithms.h>
 #include "errors.h"
 #include <x509/common.h>
-
+#include "state.h"
 
 extern mod_auth_st rsa_auth_struct;
 extern mod_auth_st dhe_rsa_auth_struct;
@@ -54,91 +54,87 @@ typedef struct {
 } gnutls_cred_map;
 
 static const gnutls_cred_map cred_mappings[] = {
-	{GNUTLS_KX_ANON_DH, GNUTLS_CRD_ANON, GNUTLS_CRD_ANON},
-	{GNUTLS_KX_ANON_ECDH, GNUTLS_CRD_ANON, GNUTLS_CRD_ANON},
-	{GNUTLS_KX_RSA, GNUTLS_CRD_CERTIFICATE, GNUTLS_CRD_CERTIFICATE},
 	{GNUTLS_KX_ECDHE_RSA, GNUTLS_CRD_CERTIFICATE,
 	 GNUTLS_CRD_CERTIFICATE},
 	{GNUTLS_KX_ECDHE_ECDSA, GNUTLS_CRD_CERTIFICATE,
 	 GNUTLS_CRD_CERTIFICATE},
+	{GNUTLS_KX_RSA, GNUTLS_CRD_CERTIFICATE, GNUTLS_CRD_CERTIFICATE},
 	{GNUTLS_KX_DHE_DSS, GNUTLS_CRD_CERTIFICATE,
 	 GNUTLS_CRD_CERTIFICATE},
 	{GNUTLS_KX_DHE_RSA, GNUTLS_CRD_CERTIFICATE,
 	 GNUTLS_CRD_CERTIFICATE},
+	{GNUTLS_KX_ECDHE_PSK, GNUTLS_CRD_PSK, GNUTLS_CRD_PSK},
 	{GNUTLS_KX_PSK, GNUTLS_CRD_PSK, GNUTLS_CRD_PSK},
 	{GNUTLS_KX_DHE_PSK, GNUTLS_CRD_PSK, GNUTLS_CRD_PSK},
 	{GNUTLS_KX_RSA_PSK, GNUTLS_CRD_PSK, GNUTLS_CRD_CERTIFICATE},
-	{GNUTLS_KX_ECDHE_PSK, GNUTLS_CRD_PSK, GNUTLS_CRD_PSK},
 	{GNUTLS_KX_SRP, GNUTLS_CRD_SRP, GNUTLS_CRD_SRP},
 	{GNUTLS_KX_SRP_RSA, GNUTLS_CRD_SRP, GNUTLS_CRD_CERTIFICATE},
 	{GNUTLS_KX_SRP_DSS, GNUTLS_CRD_SRP, GNUTLS_CRD_CERTIFICATE},
+	{GNUTLS_KX_ANON_DH, GNUTLS_CRD_ANON, GNUTLS_CRD_ANON},
+	{GNUTLS_KX_ANON_ECDH, GNUTLS_CRD_ANON, GNUTLS_CRD_ANON},
 	{0, 0, 0}
 };
 
 #define GNUTLS_KX_MAP_LOOP(b) \
-        const gnutls_cred_map *p; \
-                for(p = cred_mappings; p->algorithm != 0; p++) { b ; }
-
-#define GNUTLS_KX_MAP_ALG_LOOP_SERVER(a) \
-                        GNUTLS_KX_MAP_LOOP( if(p->server_type == type) { a; break; })
+	const gnutls_cred_map *p; \
+		for(p = cred_mappings; p->algorithm != 0; p++) { b ; }
 
 struct gnutls_kx_algo_entry {
 	const char *name;
 	gnutls_kx_algorithm_t algorithm;
 	mod_auth_st *auth_struct;
 	bool needs_dh_params;
-	gnutls_pk_algorithm_t cert_pk; /* the type of certificate required */
 	bool false_start;
 };
 typedef struct gnutls_kx_algo_entry gnutls_kx_algo_entry;
 
 static const gnutls_kx_algo_entry _gnutls_kx_algorithms[] = {
-#if defined(ENABLE_ANON) && defined(ENABLE_DHE)
-	{"ANON-DH", GNUTLS_KX_ANON_DH, &anon_auth_struct, 1, GNUTLS_PK_UNKNOWN, 0},
-#endif
-#if defined(ENABLE_ANON) && defined(ENABLE_ECDHE)
-	{"ANON-ECDH", GNUTLS_KX_ANON_ECDH, &anon_ecdh_auth_struct, 0, GNUTLS_PK_UNKNOWN, 0},
-#endif
-	{"RSA", GNUTLS_KX_RSA, &rsa_auth_struct, 0, GNUTLS_PK_RSA, 0},
-#ifdef ENABLE_DHE
-	{"DHE-RSA", GNUTLS_KX_DHE_RSA, &dhe_rsa_auth_struct, 1, GNUTLS_PK_RSA, 1},
-	{"DHE-DSS", GNUTLS_KX_DHE_DSS, &dhe_dss_auth_struct, 1, GNUTLS_PK_DSA, 1},
-#endif
 #ifdef ENABLE_ECDHE
-	{"ECDHE-RSA", GNUTLS_KX_ECDHE_RSA, &ecdhe_rsa_auth_struct, 0, GNUTLS_PK_RSA, 1},
+	{"ECDHE-RSA", GNUTLS_KX_ECDHE_RSA, &ecdhe_rsa_auth_struct, 0, 1},
 	{"ECDHE-ECDSA", GNUTLS_KX_ECDHE_ECDSA, &ecdhe_ecdsa_auth_struct,
-	 0, GNUTLS_PK_EC, 1},
+	 0, 1},
 #endif
-#ifdef ENABLE_SRP
-	{"SRP-DSS", GNUTLS_KX_SRP_DSS, &srp_dss_auth_struct, 0, GNUTLS_PK_DSA, 0},
-	{"SRP-RSA", GNUTLS_KX_SRP_RSA, &srp_rsa_auth_struct, 0, GNUTLS_PK_RSA, 0},
-	{"SRP", GNUTLS_KX_SRP, &srp_auth_struct, 0, GNUTLS_PK_UNKNOWN, 0},
+	{"RSA", GNUTLS_KX_RSA, &rsa_auth_struct, 0, 0},
+#ifdef ENABLE_DHE
+	{"DHE-RSA", GNUTLS_KX_DHE_RSA, &dhe_rsa_auth_struct, 1, 1},
+	{"DHE-DSS", GNUTLS_KX_DHE_DSS, &dhe_dss_auth_struct, 1, 1},
 #endif
 #ifdef ENABLE_PSK
-	{"PSK", GNUTLS_KX_PSK, &psk_auth_struct, 0, GNUTLS_PK_UNKNOWN, 0},
-	{"RSA-PSK", GNUTLS_KX_RSA_PSK, &rsa_psk_auth_struct, 0, GNUTLS_PK_RSA, 0},
+	{"PSK", GNUTLS_KX_PSK, &psk_auth_struct, 0, 0},
+	{"RSA-PSK", GNUTLS_KX_RSA_PSK, &rsa_psk_auth_struct, 0, 0},
 #ifdef ENABLE_DHE
 	{"DHE-PSK", GNUTLS_KX_DHE_PSK, &dhe_psk_auth_struct,
-	 1 /* needs DHE params */, GNUTLS_PK_UNKNOWN, 0},
+	 1 /* needs DHE params */, 0},
 #endif
 #ifdef ENABLE_ECDHE
-	{"ECDHE-PSK", GNUTLS_KX_ECDHE_PSK, &ecdhe_psk_auth_struct, 0, GNUTLS_PK_UNKNOWN, 0},
+	{"ECDHE-PSK", GNUTLS_KX_ECDHE_PSK, &ecdhe_psk_auth_struct, 0, 0},
 #endif
+#endif
+#ifdef ENABLE_SRP
+	{"SRP-DSS", GNUTLS_KX_SRP_DSS, &srp_dss_auth_struct, 0, 0},
+	{"SRP-RSA", GNUTLS_KX_SRP_RSA, &srp_rsa_auth_struct, 0, 0},
+	{"SRP", GNUTLS_KX_SRP, &srp_auth_struct, 0, 0},
+#endif
+#if defined(ENABLE_ANON) && defined(ENABLE_DHE)
+	{"ANON-DH", GNUTLS_KX_ANON_DH, &anon_auth_struct, 1, 0},
+#endif
+#if defined(ENABLE_ANON) && defined(ENABLE_ECDHE)
+	{"ANON-ECDH", GNUTLS_KX_ANON_ECDH, &anon_ecdh_auth_struct, 0, 0},
 #endif
 	/* for deprecated and legacy algorithms no longer supported, use
 	 * GNUTLS_KX_INVALID as an entry. This will make them available
 	 * as priority strings, but they will be a no-op.
 	 */
-	{"RSA-EXPORT", GNUTLS_KX_INVALID, NULL, 0, GNUTLS_PK_UNKNOWN, 0},
+	{"RSA-EXPORT", GNUTLS_KX_INVALID, NULL, 0, 0},
 	{0, 0, 0, 0, 0}
 };
 
 #define GNUTLS_KX_LOOP(b) \
-        const gnutls_kx_algo_entry *p; \
-                for(p = _gnutls_kx_algorithms; p->name != NULL; p++) { b ; }
+	const gnutls_kx_algo_entry *p; \
+		for(p = _gnutls_kx_algorithms; p->name != NULL; p++) { b ; }
 
 #define GNUTLS_KX_ALG_LOOP(a) \
-                        GNUTLS_KX_LOOP( if(p->algorithm == algorithm) { a; break; } )
+			GNUTLS_KX_LOOP( if(p->algorithm == algorithm) { a; break; } )
 
 
 /* Key EXCHANGE functions */
@@ -150,19 +146,6 @@ mod_auth_st *_gnutls_kx_auth_struct(gnutls_kx_algorithm_t algorithm)
 
 }
 
-int
-_gnutls_kx_priority(gnutls_session_t session,
-		    gnutls_kx_algorithm_t algorithm)
-{
-	unsigned int i;
-	for (i = 0; i < session->internals.priorities.kx.algorithms; i++) {
-		if (session->internals.priorities.kx.priority[i] ==
-		    algorithm)
-			return i;
-	}
-	return -1;
-}
-
 /**
  * gnutls_kx_get_name:
  * @algorithm: is a key exchange algorithm
@@ -261,7 +244,7 @@ int _gnutls_kx_is_ok(gnutls_kx_algorithm_t algorithm)
 
 bool _gnutls_kx_allows_false_start(gnutls_session_t session)
 {
-	unsigned algorithm = session->security_parameters.kx_algorithm;
+	unsigned algorithm = session->security_parameters.cs->kx_algorithm;
 	bool needs_dh = 0;
 	int bits;
 
@@ -269,18 +252,25 @@ bool _gnutls_kx_allows_false_start(gnutls_session_t session)
 	GNUTLS_KX_ALG_LOOP(ret = p->false_start; needs_dh = p->needs_dh_params);
 
 	if (ret != 0) {
+		const gnutls_group_entry_st *e;
+
+		e = get_group(session);
+
 #if defined(ENABLE_DHE) || defined(ENABLE_ANON)
 		if (needs_dh != 0) {
 			bits = gnutls_sec_param_to_pk_bits(GNUTLS_PK_DH, GNUTLS_SEC_PARAM_HIGH);
 			/* check whether sizes are sufficient */
-			if (gnutls_dh_get_prime_bits(session) < bits)
+			if (e && e->prime) {
+				if (e->prime->size*8 < (unsigned)bits)
+					ret = 0;
+			} else if (gnutls_dh_get_prime_bits(session) < bits)
 				ret = 0;
 		} else 
 #endif
 		if (algorithm == GNUTLS_KX_ECDHE_RSA || algorithm == GNUTLS_KX_ECDHE_ECDSA) {
 			bits = gnutls_sec_param_to_pk_bits(GNUTLS_PK_EC, GNUTLS_SEC_PARAM_HIGH);
 
-			if (gnutls_ecc_curve_get_size(session->security_parameters.ecc_curve) * 8 < bits)
+			if (e != NULL && gnutls_ecc_curve_get_size(e->curve) * 8 < bits)
 				ret = 0;
 		}
 	}
@@ -294,27 +284,6 @@ bool _gnutls_kx_needs_dh_params(gnutls_kx_algorithm_t algorithm)
 	return ret;
 }
 
-int _gnutls_kx_cert_pk_params(gnutls_kx_algorithm_t algorithm)
-{
-	ssize_t ret = 0;
-	GNUTLS_KX_ALG_LOOP(ret = p->cert_pk);
-	return ret;
-}
-
-/* Type to KX mappings */
-gnutls_kx_algorithm_t
-_gnutls_map_kx_get_kx(gnutls_credentials_type_t type, int server)
-{
-	gnutls_kx_algorithm_t ret = -1;
-
-	if (server) {
-		GNUTLS_KX_MAP_ALG_LOOP_SERVER(ret = p->algorithm);
-	} else {
-		GNUTLS_KX_MAP_ALG_LOOP_SERVER(ret = p->algorithm);
-	}
-	return ret;
-}
-
 /* Returns the credentials type required for this
  * Key exchange method.
  */
