@@ -30,6 +30,8 @@
 #include <gnutls/compat.h>
 #include <str_array.h>
 
+#define MAX_OCSP_RESPONSES 8
+
 typedef struct {
 	gnutls_pcert_st *cert_list;	/* a certificate chain */
 	unsigned int cert_list_length;	/* its length */
@@ -37,7 +39,9 @@ typedef struct {
 
 	gnutls_status_request_ocsp_func ocsp_func;
 	void *ocsp_func_ptr; /* corresponding OCSP response function + ptr */
-	char *ocsp_response_file; /* corresponding OCSP response file */
+
+	gnutls_ocsp_data_st ocsp_data[MAX_OCSP_RESPONSES];
+	unsigned int ocsp_data_length;
 
 	/* the private key corresponding to certificate */
 	gnutls_privkey_t pkey;
@@ -77,8 +81,9 @@ typedef struct gnutls_certificate_credentials_st {
 
 	/* It's a mess here. However we need to keep the old 3 functions
 	 * for compatibility */
-	gnutls_certificate_retrieve_function *get_cert_callback;	/* deprecated */
-	gnutls_certificate_retrieve_function2 *get_cert_callback2;
+	gnutls_certificate_retrieve_function *legacy_cert_cb1; /* deprecated */
+	gnutls_certificate_retrieve_function2 *legacy_cert_cb2;
+	gnutls_certificate_retrieve_function3 *get_cert_callback3;
 
 	gnutls_certificate_verify_function *verify_callback;
 
@@ -91,11 +96,6 @@ typedef struct gnutls_certificate_credentials_st {
 	void *glob_ocsp_func_ptr; /* corresponding OCSP response function */
 } certificate_credentials_st;
 
-typedef struct rsa_info_st {
-	gnutls_datum_t modulus;
-	gnutls_datum_t exponent;
-} rsa_info_st;
-
 /* This is the information we keep for the peer
  * certificate.
  */
@@ -105,18 +105,21 @@ typedef struct cert_auth_info_st {
 	 */
 	dh_info_st dh;
 
-	gnutls_datum_t *raw_certificate_list;	/* holds the raw certificate of the
-						 * peer.
-						 */
-	unsigned int ncerts;	/* holds the size of the list above */
+	/* we store the peer's OCSP responses received during
+	 * this session. */
+	gnutls_datum_t *raw_ocsp_list;
+	unsigned int nocsp;
+
+	/* we store the peer's certificates received during
+	 * this ession */
+	gnutls_datum_t *raw_certificate_list;
+	unsigned int ncerts;
 
 	gnutls_certificate_type_t cert_type;
 } *cert_auth_info_t;
 
 typedef struct cert_auth_info_st cert_auth_info_st;
 
-void _gnutls_free_rsa_info(rsa_info_st * rsa);
-
 /* AUTH X509 functions */
 int _gnutls_gen_cert_server_crt(gnutls_session_t, gnutls_buffer_st *);
 int _gnutls_gen_cert_client_crt(gnutls_session_t, gnutls_buffer_st *);
@@ -130,6 +133,13 @@ int _gnutls_get_selected_cert(gnutls_session_t session,
 			      int *apr_cert_list_length,
 			      gnutls_privkey_t * apr_pkey);
 
+int
+_gnutls_select_client_cert(gnutls_session_t session,
+			   const uint8_t * _data, size_t _data_size,
+			   gnutls_pk_algorithm_t * pk_algos, int pk_algos_length);
+
+int _gnutls_pcert_to_auth_info(cert_auth_info_t info, gnutls_pcert_st * certs, size_t ncerts);
+
 int
 _gnutls_server_select_cert(gnutls_session_t session, const gnutls_cipher_suite_entry_st *cs);
 void _gnutls_selected_certs_deinit(gnutls_session_t session);
