@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2000-2013 Free Software Foundation, Inc.
  * Copyright (C) 2013 Nikos Mavrogiannopoulos
- * Copyright (C) 2017 Red Hat, Inc.
+ * Copyright (C) 2017-2018 Red Hat, Inc.
  *
  * Author: Nikos Mavrogiannopoulos
  *
@@ -48,38 +48,69 @@ static int encrypt_packet(gnutls_session_t session,
 			    size_t min_pad,
 			    content_type_t _type,
 			    record_parameters_st * params);
+
 static int decrypt_packet(gnutls_session_t session,
 			    gnutls_datum_t * ciphertext,
 			    gnutls_datum_t * plain,
-			    uint8_t type,
+			    content_type_t type,
 			    record_parameters_st * params,
 			    gnutls_uint64 * sequence);
 
+static int
+decrypt_packet_tls13(gnutls_session_t session,
+		     gnutls_datum_t * ciphertext,
+		     gnutls_datum_t * plain,
+		     content_type_t *type, record_parameters_st * params,
+		     gnutls_uint64 * sequence);
+
+static int
+encrypt_packet_tls13(gnutls_session_t session,
+		     uint8_t *cipher_data, size_t cipher_size,
+		     gnutls_datum_t *plain,
+		     size_t pad_size,
+		     uint8_t type,
+		     record_parameters_st *params);
+
 /* returns ciphertext which contains the headers too. This also
  * calculates the size in the header field.
  * 
  */
 int
 _gnutls_encrypt(gnutls_session_t session,
-		const uint8_t * data, size_t data_size,
+		const uint8_t *data, size_t data_size,
 		size_t min_pad,
-		mbuffer_st * bufel,
-		content_type_t type, record_parameters_st * params)
+		mbuffer_st *bufel,
+		content_type_t type, record_parameters_st *params)
 {
 	gnutls_datum_t plaintext;
+	const version_entry_st *vers = get_version(session);
 	int ret;
 
 	plaintext.data = (uint8_t *) data;
 	plaintext.size = data_size;
 
-	ret =
-	    encrypt_packet(session,
-				     _mbuffer_get_udata_ptr(bufel),
-				     _mbuffer_get_udata_size
-				     (bufel), &plaintext, min_pad, type,
-				     params);
-	if (ret < 0)
-		return gnutls_assert_val(ret);
+	if (vers && vers->tls13_sem) {
+		/* it fills the header, as it is included in the authenticated
+		 * data of the AEAD cipher. */
+		ret =
+		    encrypt_packet_tls13(session,
+					 _mbuffer_get_udata_ptr(bufel),
+					 _mbuffer_get_udata_size(bufel),
+					 &plaintext, min_pad, type,
+					 params);
+		if (ret < 0)
+			return gnutls_assert_val(ret);
+	} else {
+		ret =
+		    encrypt_packet(session,
+				   _mbuffer_get_udata_ptr(bufel),
+				   _mbuffer_get_udata_size
+				   (bufel), &plaintext, min_pad, type,
+				   params);
+		if (ret < 0)
+			return gnutls_assert_val(ret);
+
+	}
 
 	if (IS_DTLS(session))
 		_gnutls_write_uint16(ret,
@@ -103,20 +134,28 @@ _gnutls_encrypt(gnutls_session_t session,
  */
 int
 _gnutls_decrypt(gnutls_session_t session,
-		gnutls_datum_t * ciphertext,
-		gnutls_datum_t * output,
-		content_type_t type,
-		record_parameters_st * params, gnutls_uint64 * sequence)
+		gnutls_datum_t *ciphertext,
+		gnutls_datum_t *output,
+		content_type_t *type,
+		record_parameters_st *params,
+		gnutls_uint64 *sequence)
 {
 	int ret;
+	const version_entry_st *vers = get_version(session);
 
 	if (ciphertext->size == 0)
 		return 0;
 
-	ret =
-	    decrypt_packet(session, ciphertext,
-				     output, type, params,
-				     sequence);
+	if (vers && vers->tls13_sem)
+		ret =
+		    decrypt_packet_tls13(session, ciphertext,
+					 output, type, params,
+					 sequence);
+	else
+		ret =
+		    decrypt_packet(session, ciphertext,
+				   output, *type, params,
+				   sequence);
 	if (ret < 0)
 		return gnutls_assert_val(ret);
 
@@ -170,14 +209,12 @@ calc_enc_length_stream(gnutls_session_t session, int data_size,
 	return length;
 }
 
-#define MAX_PREAMBLE_SIZE 16
-
 /* generates the authentication data (data to be hashed only
  * and are not to be sent). Returns their size.
  */
-static inline int
-make_preamble(uint8_t * uint64_data, uint8_t type, unsigned int length,
-	      const version_entry_st * ver, uint8_t * preamble)
+int
+_gnutls_make_preamble(uint8_t * uint64_data, uint8_t type, unsigned int length,
+		      const version_entry_st * ver, uint8_t preamble[MAX_PREAMBLE_SIZE])
 {
 	uint8_t *p = preamble;
 	uint16_t c_length;
@@ -220,15 +257,15 @@ encrypt_packet(gnutls_session_t session,
 	uint8_t preamble[MAX_PREAMBLE_SIZE];
 	int preamble_size;
 	int tag_size =
-	    _gnutls_auth_cipher_tag_len(&params->write.cipher_state);
+	    _gnutls_auth_cipher_tag_len(&params->write.ctx.tls12);
 	int blocksize = _gnutls_cipher_get_block_size(params->cipher);
 	unsigned algo_type = _gnutls_cipher_type(params->cipher);
 	uint8_t *data_ptr, *full_cipher_ptr;
 	const version_entry_st *ver = get_version(session);
 	int explicit_iv = _gnutls_version_has_explicit_iv(ver);
 	int auth_cipher =
-	    _gnutls_auth_cipher_is_aead(&params->write.cipher_state);
-	uint8_t nonce[MAX_CIPHER_BLOCK_SIZE];
+	    _gnutls_auth_cipher_is_aead(&params->write.ctx.tls12);
+	uint8_t nonce[MAX_CIPHER_IV_SIZE];
 	unsigned imp_iv_size = 0, exp_iv_size = 0;
 	bool etm = 0;
 
@@ -286,7 +323,7 @@ encrypt_packet(gnutls_session_t session,
 			 */
 			memcpy(data_ptr, nonce, blocksize);
 			_gnutls_auth_cipher_setiv(&params->write.
-						  cipher_state, data_ptr,
+						  ctx.tls12, data_ptr,
 						  blocksize);
 
 			/*data_ptr += blocksize;*/
@@ -296,9 +333,7 @@ encrypt_packet(gnutls_session_t session,
 		if (params->cipher->xor_nonce == 0) {
 			/* Values in AEAD are pretty fixed in TLS 1.2 for 128-bit block
 			 */
-			 if (params->write.IV.data == NULL
-			    || params->write.IV.size !=
-			    imp_iv_size)
+			 if (params->write.iv_size != imp_iv_size)
 				return
 				    gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
 
@@ -306,8 +341,8 @@ encrypt_packet(gnutls_session_t session,
 			 * write.sequence_number (It is a MAY on RFC 5288), and safer
 			 * as it will never reuse a value.
 			 */
-			memcpy(nonce, params->write.IV.data,
-			       params->write.IV.size);
+			memcpy(nonce, params->write.iv,
+			       params->write.iv_size);
 			memcpy(&nonce[imp_iv_size],
 			       UINT64DATA(params->write.sequence_number),
 			       8);
@@ -318,14 +353,14 @@ encrypt_packet(gnutls_session_t session,
 			/*data_ptr += exp_iv_size;*/
 			cipher_data += exp_iv_size;
 		} else { /* XOR nonce with IV */
-			if (unlikely(params->write.IV.size != 12 || imp_iv_size != 12 || exp_iv_size != 0))
+			if (unlikely(params->write.iv_size != 12 || imp_iv_size != 12 || exp_iv_size != 0))
 				return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
 
 			memset(nonce, 0, 4);
 			memcpy(&nonce[4],
 			       UINT64DATA(params->write.sequence_number), 8);
 
-			memxor(nonce, params->write.IV.data, 12);
+			memxor(nonce, params->write.iv, 12);
 		}
 	}
 
@@ -335,13 +370,13 @@ encrypt_packet(gnutls_session_t session,
 		ret = plain->size;
 
 	preamble_size =
-	    make_preamble(UINT64DATA(params->write.sequence_number),
-			  type, ret, ver, preamble);
+	    _gnutls_make_preamble(UINT64DATA(params->write.sequence_number),
+				  type, ret, ver, preamble);
 
 	if (algo_type == CIPHER_BLOCK || algo_type == CIPHER_STREAM) {
 		/* add the authenticated data */
 		ret =
-		    _gnutls_auth_cipher_add_auth(&params->write.cipher_state,
+		    _gnutls_auth_cipher_add_auth(&params->write.ctx.tls12,
 					 preamble, preamble_size);
 		if (ret < 0)
 			return gnutls_assert_val(ret);
@@ -349,7 +384,7 @@ encrypt_packet(gnutls_session_t session,
 		if (etm && explicit_iv) {
 			/* In EtM we need to hash the IV as well */
 			ret =
-			    _gnutls_auth_cipher_add_auth(&params->write.cipher_state,
+			    _gnutls_auth_cipher_add_auth(&params->write.ctx.tls12,
 						 full_cipher_ptr, blocksize);
 			if (ret < 0)
 				return gnutls_assert_val(ret);
@@ -358,14 +393,14 @@ encrypt_packet(gnutls_session_t session,
 		/* Actual encryption.
 		 */
 		ret =
-		    _gnutls_auth_cipher_encrypt2_tag(&params->write.cipher_state,
+		    _gnutls_auth_cipher_encrypt2_tag(&params->write.ctx.tls12,
 						     plain->data,
 						     plain->size, cipher_data,
 						     cipher_size, pad);
 		if (ret < 0)
 			return gnutls_assert_val(ret);
 	} else { /* AEAD */
-		ret = _gnutls_aead_cipher_encrypt(&params->write.cipher_state.cipher,
+		ret = _gnutls_aead_cipher_encrypt(&params->write.ctx.tls12.cipher,
 						  nonce, imp_iv_size + exp_iv_size,
 						  preamble, preamble_size,
 						  tag_size,
@@ -378,66 +413,128 @@ encrypt_packet(gnutls_session_t session,
 	return length;
 }
 
-static void dummy_wait(record_parameters_st * params,
-		       gnutls_datum_t * plaintext, unsigned pad_failed,
-		       unsigned int pad, unsigned total)
+static int
+encrypt_packet_tls13(gnutls_session_t session,
+		     uint8_t *cipher_data, size_t cipher_size,
+		     gnutls_datum_t *plain,
+		     size_t pad_size,
+		     uint8_t type,
+		     record_parameters_st *params)
 {
-	/* this hack is only needed on CBC ciphers */
-	if (_gnutls_cipher_type(params->cipher) == CIPHER_BLOCK) {
-		unsigned len;
+	int ret;
+	unsigned int tag_size = params->write.aead_tag_size;
+	const version_entry_st *ver = get_version(session);
+	uint8_t nonce[MAX_CIPHER_IV_SIZE];
+	unsigned iv_size = 0;
+	ssize_t max, total;
+	uint8_t aad[5];
+	giovec_t auth_iov[1];
+	giovec_t iov[2];
 
-		/* force an additional hash compression function evaluation to prevent timing 
-		 * attacks that distinguish between wrong-mac + correct pad, from wrong-mac + incorrect pad.
-		 */
-		if (pad_failed == 0 && pad > 0) {
-			len = _gnutls_mac_block_size(params->mac);
-			if (len > 0) {
-				/* This is really specific to the current hash functions.
-				 * It should be removed once a protocol fix is in place.
-				 */
-				if ((pad + total) % len > len - 9
-				    && total % len <= len - 9) {
-					if (len < plaintext->size)
-						_gnutls_auth_cipher_add_auth
-						    (&params->read.
-						     cipher_state,
-						     plaintext->data, len);
-					else
-						_gnutls_auth_cipher_add_auth
-						    (&params->read.
-						     cipher_state,
-						     plaintext->data,
-						     plaintext->size);
-				}
-			}
-		}
+	if (unlikely(ver == NULL))
+		return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
+
+	_gnutls_hard_log("ENC[%p]: cipher: %s, MAC: %s, Epoch: %u\n",
+			 session, _gnutls_cipher_get_name(params->cipher),
+			 _gnutls_mac_get_name(params->mac),
+			 (unsigned int) params->epoch);
+
+	iv_size = params->write.iv_size;
+
+	if (params->cipher->id == GNUTLS_CIPHER_NULL) {
+		if (cipher_size < plain->size+1)
+			return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
+		memcpy(cipher_data, plain->data, plain->size);
+		return plain->size;
 	}
+
+	memcpy(nonce, params->write.iv, iv_size);
+	memxor(&nonce[iv_size-8], UINT64DATA(params->write.sequence_number), 8);
+
+	max = MAX_RECORD_SEND_SIZE(session);
+
+	/* make TLS 1.3 form of data */
+	total = plain->size + 1 + pad_size;
+
+	/* check whether padding would exceed max */
+	if (total > max) {
+		if (unlikely(max < (ssize_t)plain->size+1))
+			return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
+
+		pad_size = max - plain->size - 1;
+		total = max;
+	}
+
+	/* create authenticated data header */
+	aad[0] = GNUTLS_APPLICATION_DATA;
+	aad[1] = 0x03;
+	aad[2] = 0x03;
+	_gnutls_write_uint16(total+tag_size, &aad[3]);
+
+	auth_iov[0].iov_base = aad;
+	auth_iov[0].iov_len = sizeof(aad);
+
+	iov[0].iov_base = plain->data;
+	iov[0].iov_len = plain->size;
+
+	if (pad_size || (session->internals.flags & GNUTLS_SAFE_PADDING_CHECK)) {
+		uint8_t *pad = gnutls_calloc(1, 1+pad_size);
+		if (pad == NULL)
+			return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
+
+		pad[0] = type;
+
+		iov[1].iov_base = pad;
+		iov[1].iov_len = 1+pad_size;
+
+		ret = gnutls_aead_cipher_encryptv(&params->write.ctx.aead,
+						  nonce, iv_size,
+						  auth_iov, 1,
+						  tag_size,
+						  iov, 2,
+						  cipher_data, &cipher_size);
+		gnutls_free(pad);
+	} else {
+		iov[1].iov_base = &type;
+		iov[1].iov_len = 1;
+
+		ret = gnutls_aead_cipher_encryptv(&params->write.ctx.aead,
+						  nonce, iv_size,
+						  auth_iov, 1,
+						  tag_size,
+						  iov, 2,
+						  cipher_data, &cipher_size);
+	}
+
+	if (ret < 0)
+		return gnutls_assert_val(ret);
+
+	return cipher_size;
 }
 
+
 /* Deciphers the ciphertext packet, and puts the result to plain.
  * Returns the actual plaintext packet size.
  */
 static int
 decrypt_packet(gnutls_session_t session,
-			 gnutls_datum_t * ciphertext,
-			 gnutls_datum_t * plain,
-			 uint8_t type, record_parameters_st * params,
-			 gnutls_uint64 * sequence)
+		gnutls_datum_t * ciphertext,
+		gnutls_datum_t * plain,
+		content_type_t type, record_parameters_st * params,
+		gnutls_uint64 * sequence)
 {
 	uint8_t tag[MAX_HASH_SIZE];
-	uint8_t nonce[MAX_CIPHER_BLOCK_SIZE];
+	uint8_t nonce[MAX_CIPHER_IV_SIZE];
 	const uint8_t *tag_ptr = NULL;
-	unsigned int pad = 0, i;
+	unsigned int pad = 0;
 	int length, length_to_decrypt;
 	uint16_t blocksize;
 	int ret;
-	unsigned int tmp_pad_failed = 0;
-	unsigned int pad_failed = 0;
 	uint8_t preamble[MAX_PREAMBLE_SIZE];
 	unsigned int preamble_size = 0;
 	const version_entry_st *ver = get_version(session);
 	unsigned int tag_size =
-	    _gnutls_auth_cipher_tag_len(&params->read.cipher_state);
+	    _gnutls_auth_cipher_tag_len(&params->read.ctx.tls12);
 	unsigned int explicit_iv = _gnutls_version_has_explicit_iv(ver);
 	unsigned imp_iv_size, exp_iv_size;
 	unsigned cipher_type = _gnutls_cipher_type(params->cipher);
@@ -458,24 +555,24 @@ decrypt_packet(gnutls_session_t session,
 		if (unlikely(ciphertext->size < tag_size))
 			return gnutls_assert_val(GNUTLS_E_UNEXPECTED_PACKET_LENGTH);
 
-		preamble_size = make_preamble(UINT64DATA(*sequence),
-					type, ciphertext->size-tag_size,
-					ver, preamble);
+		preamble_size = _gnutls_make_preamble(UINT64DATA(*sequence),
+						      type, ciphertext->size-tag_size,
+						      ver, preamble);
 
 		ret = _gnutls_auth_cipher_add_auth(&params->read.
-						   cipher_state, preamble,
+						   ctx.tls12, preamble,
 						   preamble_size);
 		if (unlikely(ret < 0))
 			return gnutls_assert_val(ret);
 
 		ret = _gnutls_auth_cipher_add_auth(&params->read.
-						   cipher_state,
+						   ctx.tls12,
 						   ciphertext->data,
 						   ciphertext->size-tag_size);
 		if (unlikely(ret < 0))
 			return gnutls_assert_val(ret);
 
-		ret = _gnutls_auth_cipher_tag(&params->read.cipher_state, tag, tag_size);
+		ret = _gnutls_auth_cipher_tag(&params->read.ctx.tls12, tag, tag_size);
 		if (unlikely(ret < 0))
 			return gnutls_assert_val(ret);
 		
@@ -493,7 +590,7 @@ decrypt_packet(gnutls_session_t session,
 		 * only stream ciphers.
 		 */
 		if (unlikely(_gnutls_auth_cipher_is_aead(&params->read.
-						   cipher_state) == 0))
+						   ctx.tls12) == 0))
 			return gnutls_assert_val(GNUTLS_E_DECRYPTION_FAILED);
 
 
@@ -503,13 +600,11 @@ decrypt_packet(gnutls_session_t session,
 		if (params->cipher->xor_nonce == 0) {
 			/* Values in AEAD are pretty fixed in TLS 1.2 for 128-bit block
 			 */
-			 if (unlikely
-			    (params->read.IV.data == NULL
-			     || params->read.IV.size != 4))
+			 if (unlikely(params->read.iv_size != 4))
 				return
 				    gnutls_assert_val(GNUTLS_E_DECRYPTION_FAILED);
 
-			memcpy(nonce, params->read.IV.data,
+			memcpy(nonce, params->read.iv,
 			       imp_iv_size);
 
 			memcpy(&nonce[imp_iv_size],
@@ -518,13 +613,13 @@ decrypt_packet(gnutls_session_t session,
 			ciphertext->data += exp_iv_size;
 			ciphertext->size -= exp_iv_size;
 		} else { /* XOR nonce with IV */
-			if (unlikely(params->read.IV.size != 12 || imp_iv_size != 12 || exp_iv_size != 0))
+			if (unlikely(params->read.iv_size != 12 || imp_iv_size != 12 || exp_iv_size != 0))
 				return gnutls_assert_val(GNUTLS_E_DECRYPTION_FAILED);
 
 			memset(nonce, 0, 4);
 			memcpy(&nonce[4], UINT64DATA(*sequence), 8);
 
-			memxor(nonce, params->read.IV.data, 12);
+			memxor(nonce, params->read.iv, 12);
 		}
 
 		length =
@@ -536,8 +631,8 @@ decrypt_packet(gnutls_session_t session,
 		 * MAC.
 		 */
 		preamble_size =
-		    make_preamble(UINT64DATA(*sequence), type,
-				  length, ver, preamble);
+		    _gnutls_make_preamble(UINT64DATA(*sequence), type,
+					  length, ver, preamble);
 
 
 		if (unlikely
@@ -550,7 +645,7 @@ decrypt_packet(gnutls_session_t session,
 			    gnutls_assert_val(GNUTLS_E_DECRYPTION_FAILED);
 		}
 
-		ret = _gnutls_aead_cipher_decrypt(&params->read.cipher_state.cipher,
+		ret = _gnutls_aead_cipher_decrypt(&params->read.ctx.tls12.cipher,
 						  nonce, exp_iv_size + imp_iv_size,
 						  preamble, preamble_size,
 						  tag_size,
@@ -576,12 +671,12 @@ decrypt_packet(gnutls_session_t session,
 		 * MAC.
 		 */
 		preamble_size =
-		    make_preamble(UINT64DATA(*sequence), type,
-				  length, ver, preamble);
+		    _gnutls_make_preamble(UINT64DATA(*sequence), type,
+					  length, ver, preamble);
 
 		ret =
 		    _gnutls_auth_cipher_add_auth(&params->read.
-						 cipher_state, preamble,
+						 ctx.tls12, preamble,
 						 preamble_size);
 		if (unlikely(ret < 0))
 			return gnutls_assert_val(ret);
@@ -598,7 +693,7 @@ decrypt_packet(gnutls_session_t session,
 
 		ret =
 		    _gnutls_auth_cipher_decrypt2(&params->read.
-						 cipher_state,
+						 ctx.tls12,
 						 ciphertext->data,
 						 length_to_decrypt,
 						 plain->data,
@@ -607,6 +702,17 @@ decrypt_packet(gnutls_session_t session,
 		if (unlikely(ret < 0))
 			return gnutls_assert_val(ret);
 
+		ret =
+		    _gnutls_auth_cipher_tag(&params->read.ctx.tls12, tag,
+					    tag_size);
+		if (unlikely(ret < 0))
+			return gnutls_assert_val(ret);
+
+		if (unlikely
+		    (gnutls_memcmp(tag, tag_ptr, tag_size) != 0)) {
+			return gnutls_assert_val(GNUTLS_E_DECRYPTION_FAILED);
+		}
+
 		break;
 	case CIPHER_BLOCK:
 		if (unlikely(ciphertext->size < blocksize))
@@ -626,7 +732,7 @@ decrypt_packet(gnutls_session_t session,
 		 */
 		if (explicit_iv) {
 			_gnutls_auth_cipher_setiv(&params->read.
-						  cipher_state,
+						  ctx.tls12,
 						  ciphertext->data,
 						  blocksize);
 
@@ -650,7 +756,7 @@ decrypt_packet(gnutls_session_t session,
 
 		if (etm == 0) {
 			ret =
-			    _gnutls_cipher_decrypt2(&params->read.cipher_state.
+			    _gnutls_cipher_decrypt2(&params->read.ctx.tls12.
 						    cipher, ciphertext->data,
 						    ciphertext->size,
 						    plain->data,
@@ -658,59 +764,16 @@ decrypt_packet(gnutls_session_t session,
 			if (unlikely(ret < 0))
 				return gnutls_assert_val(ret);
 
-			pad = plain->data[ciphertext->size - 1];	/* pad */
-
-			/* Check the pading bytes (TLS 1.x). 
-			 * Note that we access all 256 bytes of ciphertext for padding check
-			 * because there is a timing channel in that memory access (in certain CPUs).
-			 */
-#ifdef ENABLE_SSL3
-			if (ver->id != GNUTLS_SSL3)
-#endif
-				for (i = 2; i <= MIN(256, ciphertext->size); i++) {
-					tmp_pad_failed |=
-					    (plain->
-					     data[ciphertext->size - i] != pad);
-					pad_failed |=
-					    ((i <= (1 + pad)) & (tmp_pad_failed));
-				}
-
-			if (unlikely
-			    (pad_failed != 0
-			     || (1 + pad > ((int) ciphertext->size - tag_size)))) {
-				/* We do not fail here. We check below for the
-				 * the pad_failed. If zero means success.
-				 */
-				pad_failed = 1;
-				pad = 0;
-			}
-
-			length = ciphertext->size - tag_size - pad - 1;
-			tag_ptr = &plain->data[length];
-
-			/* Pass the type, version, length and plain through
-			 * MAC.
-			 */
-			preamble_size =
-			    make_preamble(UINT64DATA(*sequence), type,
-					  length, ver, preamble);
-
-			ret =
-			    _gnutls_auth_cipher_add_auth(&params->read.
-							 cipher_state, preamble,
-							 preamble_size);
+			ret = cbc_mac_verify(session, params, preamble, type,
+					     sequence, plain->data, ciphertext->size,
+					     tag_size);
 			if (unlikely(ret < 0))
 				return gnutls_assert_val(ret);
 
-			ret =
-			    _gnutls_auth_cipher_add_auth(&params->read.
-							 cipher_state,
-							 plain->data, length);
-			if (unlikely(ret < 0))
-				return gnutls_assert_val(ret);
+			length = ret;
 		} else { /* EtM */
 			ret =
-			    _gnutls_cipher_decrypt2(&params->read.cipher_state.
+			    _gnutls_cipher_decrypt2(&params->read.ctx.tls12.
 						    cipher, ciphertext->data,
 						    ciphertext->size - tag_size,
 						    plain->data,
@@ -729,29 +792,100 @@ decrypt_packet(gnutls_session_t session,
 		return gnutls_assert_val(GNUTLS_E_DECRYPTION_FAILED);
 	}
 
-	/* STREAM or BLOCK arrive here */
-	if (etm == 0) {
-		ret =
-		    _gnutls_auth_cipher_tag(&params->read.cipher_state, tag,
-					    tag_size);
-		if (unlikely(ret < 0))
-			return gnutls_assert_val(ret);
-
-		/* Here there could be a timing leakage in CBC ciphersuites that
-		 * could be exploited if the cost of a successful memcmp is high. 
-		 * A constant time memcmp would help there, but it is not easy to maintain
-		 * against compiler optimizations. Currently we rely on the fact that
-		 * a memcmp comparison is negligible over the crypto operations.
-		 */
-		if (unlikely
-		    (gnutls_memcmp(tag, tag_ptr, tag_size) != 0 || pad_failed != 0)) {
-			/* HMAC was not the same. */
-			dummy_wait(params, plain, pad_failed, pad,
-				   length + preamble_size);
-
-			return gnutls_assert_val(GNUTLS_E_DECRYPTION_FAILED);
+
+	return length;
+}
+
+static int
+decrypt_packet_tls13(gnutls_session_t session,
+		     gnutls_datum_t *ciphertext,
+		     gnutls_datum_t *plain,
+		     content_type_t *type, record_parameters_st *params,
+		     gnutls_uint64 *sequence)
+{
+	uint8_t nonce[MAX_CIPHER_IV_SIZE];
+	size_t length, length_to_decrypt;
+	int ret;
+	const version_entry_st *ver = get_version(session);
+	unsigned int tag_size = params->read.aead_tag_size;
+	unsigned iv_size;
+	unsigned j;
+	volatile unsigned length_set;
+	uint8_t aad[5];
+
+	if (unlikely(ver == NULL))
+		return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
+
+	if (params->cipher->id == GNUTLS_CIPHER_NULL) {
+		if (plain->size < ciphertext->size)
+			return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
+
+		length = ciphertext->size;
+		memcpy(plain->data, ciphertext->data, length);
+
+		return length;
+	}
+
+	iv_size = _gnutls_cipher_get_iv_size(params->cipher);
+
+	/* The way AEAD ciphers are defined in RFC5246, it allows
+	 * only stream ciphers.
+	 */
+	if (unlikely(ciphertext->size < tag_size))
+		return gnutls_assert_val(GNUTLS_E_DECRYPTION_FAILED);
+
+	if (unlikely(params->read.iv_size != iv_size || iv_size < 8))
+		return gnutls_assert_val(GNUTLS_E_DECRYPTION_FAILED);
+
+	memcpy(nonce, params->read.iv, params->read.iv_size);
+	memxor(&nonce[iv_size-8], UINT64DATA(*sequence), 8);
+
+	length =
+	    ciphertext->size - tag_size;
+
+	length_to_decrypt = ciphertext->size;
+
+	if (unlikely
+	    ((unsigned) length_to_decrypt > plain->size)) {
+			_gnutls_audit_log(session,
+				  "Received %u bytes, while expecting less than %u\n",
+				  (unsigned int) length_to_decrypt,
+				  (unsigned int) plain->size);
+		return
+		    gnutls_assert_val(GNUTLS_E_DECRYPTION_FAILED);
+	}
+
+	aad[0] = GNUTLS_APPLICATION_DATA;
+	aad[1] = 0x03;
+	aad[2] = 0x03;
+	_gnutls_write_uint16(ciphertext->size, &aad[3]);
+
+	ret = gnutls_aead_cipher_decrypt(&params->read.ctx.aead,
+					 nonce, iv_size,
+					 aad, sizeof(aad),
+					 tag_size,
+					 ciphertext->data, length_to_decrypt,
+					 plain->data, &length);
+	if (unlikely(ret < 0))
+		return gnutls_assert_val(ret);
+
+	length_set = 0;
+
+	/* now figure the actual data size. We intentionally iterate through all data,
+	 * to avoid leaking the padding length due to timing differences in processing.
+	 */
+	for (j=length;j>0;j--) {
+		if (plain->data[j-1]!=0 && length_set == 0) {
+			*type = plain->data[j-1];
+			length = j-1;
+			length_set = 1;
+			if (!(session->internals.flags & GNUTLS_SAFE_PADDING_CHECK))
+				break;
 		}
 	}
+
+	if (!length_set)
+		return gnutls_assert_val(GNUTLS_E_DECRYPTION_FAILED);
 
 	return length;
 }
