@@ -34,12 +34,15 @@ static int _gnutls_alpn_send_params(gnutls_session_t session,
 static void _gnutls_alpn_deinit_data(gnutls_ext_priv_data_t priv);
 
 
-const extension_entry_st ext_mod_alpn = {
+const hello_ext_entry_st ext_mod_alpn = {
 	.name = "ALPN",
-	.type = GNUTLS_EXTENSION_ALPN,
+	.tls_id = 16,
+	.gid = GNUTLS_EXTENSION_ALPN,
 	/* this extension must be parsed even on resumption */
 	.parse_type = GNUTLS_EXT_MANDATORY,
-
+	.validity = GNUTLS_EXT_FLAG_TLS | GNUTLS_EXT_FLAG_DTLS |
+		    GNUTLS_EXT_FLAG_CLIENT_HELLO | GNUTLS_EXT_FLAG_EE |
+		    GNUTLS_EXT_FLAG_TLS12_SERVER_HELLO,
 	.recv_func = _gnutls_alpn_recv_params,
 	.send_func = _gnutls_alpn_send_params,
 	.deinit_func = _gnutls_alpn_deinit_data,
@@ -60,7 +63,7 @@ _gnutls_alpn_recv_params(gnutls_session_t session,
 	int selected_protocol_index;
 
 	ret =
-	    _gnutls_ext_get_session_data(session, GNUTLS_EXTENSION_ALPN,
+	    _gnutls_hello_ext_get_priv(session, GNUTLS_EXTENSION_ALPN,
 					 &epriv);
 	if (ret < 0)
 		return 0;
@@ -148,7 +151,7 @@ _gnutls_alpn_send_params(gnutls_session_t session,
 	gnutls_ext_priv_data_t epriv;
 
 	ret =
-	    _gnutls_ext_get_session_data(session, GNUTLS_EXTENSION_ALPN,
+	    _gnutls_hello_ext_get_priv(session, GNUTLS_EXTENSION_ALPN,
 					 &epriv);
 	if (ret < 0)
 		return 0;
@@ -237,7 +240,7 @@ gnutls_alpn_get_selected_protocol(gnutls_session_t session,
 	gnutls_ext_priv_data_t epriv;
 
 	ret =
-	    _gnutls_ext_get_session_data(session, GNUTLS_EXTENSION_ALPN,
+	    _gnutls_hello_ext_get_priv(session, GNUTLS_EXTENSION_ALPN,
 					 &epriv);
 	if (ret < 0) {
 		gnutls_assert();
@@ -286,7 +289,7 @@ gnutls_alpn_set_protocols(gnutls_session_t session,
 	unsigned i;
 
 	ret =
-	    _gnutls_ext_get_session_data(session, GNUTLS_EXTENSION_ALPN,
+	    _gnutls_hello_ext_get_priv(session, GNUTLS_EXTENSION_ALPN,
 					 &epriv);
 	if (ret < 0) {
 		priv = gnutls_calloc(1, sizeof(*priv));
@@ -295,7 +298,7 @@ gnutls_alpn_set_protocols(gnutls_session_t session,
 			return GNUTLS_E_MEMORY_ERROR;
 		}
 		epriv = priv;
-		_gnutls_ext_set_session_data(session,
+		_gnutls_hello_ext_set_priv(session,
 					     GNUTLS_EXTENSION_ALPN, epriv);
 	} else
 		priv = epriv;
