@@ -23,8 +23,8 @@
 #ifndef EXT_ETM_H
 #define EXT_ETM_H
 
-#include <gnutls_extensions.h>
+#include <hello_ext.h>
 
-extern extension_entry_st ext_mod_etm;
+extern const hello_ext_entry_st ext_mod_etm;
 
 #endif
