@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014 Red Hat, Inc.
+ * Copyright (C) 2014-2017 Red Hat, Inc.
  *
  * Author: Nikos Mavrogiannopoulos
  *
@@ -20,13 +20,13 @@
  *
  */
 
-/* This file contains the code for the Max Record Size TLS extension.
+/* This file contains the code for the RFC7627 (ext master secret) TLS extension.
  */
 
 #include "gnutls_int.h"
 #include "errors.h"
 #include "num.h"
-#include <extensions.h>
+#include <hello_ext.h>
 #include <ext/ext_master_secret.h>
 
 static int _gnutls_ext_master_secret_recv_params(gnutls_session_t session,
@@ -35,18 +35,31 @@ static int _gnutls_ext_master_secret_recv_params(gnutls_session_t session,
 static int _gnutls_ext_master_secret_send_params(gnutls_session_t session,
 					  gnutls_buffer_st * extdata);
 
-const extension_entry_st ext_mod_ext_master_secret = {
-	.name = "EXT MASTER SECRET",
-	.type = GNUTLS_EXTENSION_EXT_MASTER_SECRET,
+const hello_ext_entry_st ext_mod_ext_master_secret = {
+	.name = "Extended Master Secret",
+	.tls_id = 23,
+	.gid = GNUTLS_EXTENSION_EXT_MASTER_SECRET,
 	.parse_type = GNUTLS_EXT_MANDATORY,
-
+	.validity = GNUTLS_EXT_FLAG_TLS|GNUTLS_EXT_FLAG_DTLS | GNUTLS_EXT_FLAG_CLIENT_HELLO |
+		    GNUTLS_EXT_FLAG_TLS12_SERVER_HELLO,
 	.recv_func = _gnutls_ext_master_secret_recv_params,
 	.send_func = _gnutls_ext_master_secret_send_params,
 	.pack_func = NULL,
 	.unpack_func = NULL,
-	.deinit_func = NULL
+	.deinit_func = NULL,
+	.cannot_be_overriden = 1
 };
 
+#ifdef ENABLE_SSL3
+static inline unsigned have_only_ssl3_enabled(gnutls_session_t session)
+{
+	if (session->internals.priorities->protocol.algorithms == 1 &&
+	    session->internals.priorities->protocol.priority[0] == GNUTLS_SSL3)
+	    return 1;
+	return 0;
+}
+#endif
+
 /* 
  * In case of a server: if an EXT_MASTER_SECRET extension type is received then it
  * sets a flag into the session security parameters.
@@ -59,14 +72,15 @@ _gnutls_ext_master_secret_recv_params(gnutls_session_t session,
 	ssize_t data_size = _data_size;
 
 	if ((session->internals.flags & GNUTLS_NO_EXTENSIONS) ||
-	    session->internals.priorities.no_ext_master_secret != 0) {
+	    session->internals.no_ext_master_secret != 0) {
 		return 0;
 	}
 
 	if (data_size != 0) {
-		return gnutls_assert_val(GNUTLS_E_RECEIVED_ILLEGAL_PARAMETER);
+		return gnutls_assert_val(GNUTLS_E_UNEXPECTED_PACKET_LENGTH);
 	}
 
+#ifdef ENABLE_SSL3
 	if (session->security_parameters.entity == GNUTLS_CLIENT) {
 		const version_entry_st *ver = get_version(session);
 
@@ -75,9 +89,10 @@ _gnutls_ext_master_secret_recv_params(gnutls_session_t session,
 
 		if (ver->id != GNUTLS_SSL3)
 			session->security_parameters.ext_master_secret = 1;
-	} else {
+	/* do not enable ext master secret if SSL 3.0 is the only protocol supported by server */
+	} else if (!have_only_ssl3_enabled(session))
+#endif
 		session->security_parameters.ext_master_secret = 1;
-	}
 
 	return 0;
 }
@@ -89,15 +104,15 @@ _gnutls_ext_master_secret_send_params(gnutls_session_t session,
 			       gnutls_buffer_st * extdata)
 {
 	if ((session->internals.flags & GNUTLS_NO_EXTENSIONS) ||
-	    session->internals.priorities.no_ext_master_secret != 0) {
+	    session->internals.no_ext_master_secret != 0) {
 	    session->security_parameters.ext_master_secret = 0;
 	    return 0;
 	}
 
 	/* this function sends the client extension data */
+#ifdef ENABLE_SSL3
 	if (session->security_parameters.entity == GNUTLS_CLIENT) {
-		if (session->internals.priorities.protocol.algorithms == 1 &&
-		    session->internals.priorities.protocol.priority[0] == GNUTLS_SSL3)
+		if (have_only_ssl3_enabled(session))
 		    return 0; /* this extension isn't available for SSL 3.0 */
 
 		return GNUTLS_E_INT_RET_0;
@@ -110,7 +125,14 @@ _gnutls_ext_master_secret_send_params(gnutls_session_t session,
 			return GNUTLS_E_INT_RET_0;
 	}
 
+
 	return 0;
+#else
+	if (session->security_parameters.entity == GNUTLS_CLIENT ||
+	    session->security_parameters.ext_master_secret != 0) 
+		return GNUTLS_E_INT_RET_0;
+	return 0;
+#endif
 }
 
 /**
@@ -118,7 +140,8 @@ _gnutls_ext_master_secret_send_params(gnutls_session_t session,
  * @session: is a #gnutls_session_t type.
  *
  * Get the status of the extended master secret extension negotiation.
- * This is in accordance to draft-ietf-tls-session-hash-01
+ * This is in accordance to RFC7627. That information is also
+ * available to the more generic gnutls_session_get_flags().
  *
  * Returns: Non-zero if the negotiation was successful or zero otherwise.
  **/
