@@ -206,7 +206,7 @@ gnutls_heartbeat_ping(gnutls_session_t session, size_t data_size,
 		session->internals.hb_local_data.length = data_size;
 		session->internals.hb_state = SHB_SEND2;
 
-		/* fallthrough */
+		FALLTHROUGH;
 	case SHB_SEND2:
 		session->internals.hb_actual_retrans_timeout_ms =
 		    session->internals.hb_retrans_timeout_ms;
@@ -228,7 +228,7 @@ gnutls_heartbeat_ping(gnutls_session_t session, size_t data_size,
 		}
 
 		session->internals.hb_state = SHB_RECV;
-		/* fallthrough */
+		FALLTHROUGH;
 
 	case SHB_RECV:
 		ret =
