@@ -31,11 +31,13 @@ static int _gnutls_sr_send_params(gnutls_session_t state,
 				  gnutls_buffer_st *);
 static void _gnutls_sr_deinit_data(gnutls_ext_priv_data_t priv);
 
-const extension_entry_st ext_mod_sr = {
+const hello_ext_entry_st ext_mod_sr = {
 	.name = "Safe Renegotiation",
-	.type = GNUTLS_EXTENSION_SAFE_RENEGOTIATION,
+	.tls_id = 65281,
+	.gid = GNUTLS_EXTENSION_SAFE_RENEGOTIATION,
+	.validity = GNUTLS_EXT_FLAG_TLS | GNUTLS_EXT_FLAG_DTLS | GNUTLS_EXT_FLAG_CLIENT_HELLO |
+		    GNUTLS_EXT_FLAG_TLS12_SERVER_HELLO,
 	.parse_type = GNUTLS_EXT_MANDATORY,
-
 	.recv_func = _gnutls_sr_recv_params,
 	.send_func = _gnutls_sr_send_params,
 	.pack_func = NULL,
@@ -56,7 +58,7 @@ _gnutls_ext_sr_finished(gnutls_session_t session, void *vdata,
 		return 0;
 	}
 
-	ret = _gnutls_ext_get_session_data(session,
+	ret = _gnutls_hello_ext_get_priv(session,
 					   GNUTLS_EXTENSION_SAFE_RENEGOTIATION,
 					   &epriv);
 	if (ret < 0) {
@@ -101,7 +103,7 @@ int _gnutls_ext_sr_verify(gnutls_session_t session)
 		return 0;
 	}
 
-	ret = _gnutls_ext_get_session_data(session,
+	ret = _gnutls_hello_ext_get_priv(session,
 					   GNUTLS_EXTENSION_SAFE_RENEGOTIATION,
 					   &epriv);
 	if (ret >= 0)
@@ -201,7 +203,7 @@ int _gnutls_ext_sr_recv_cs(gnutls_session_t session)
 	sr_ext_st *priv;
 	gnutls_ext_priv_data_t epriv;
 
-	ret = _gnutls_ext_get_session_data(session,
+	ret = _gnutls_hello_ext_get_priv(session,
 					   GNUTLS_EXTENSION_SAFE_RENEGOTIATION,
 					   &epriv);
 	if (ret < 0) {
@@ -220,10 +222,10 @@ int _gnutls_ext_sr_recv_cs(gnutls_session_t session)
 
 	priv->safe_renegotiation_received = 1;
 	priv->connection_using_safe_renegotiation = 1;
-	_gnutls_extension_list_add_sr(session);
+	_gnutls_hello_ext_save_sr(session);
 
 	if (set != 0)
-		_gnutls_ext_set_session_data(session,
+		_gnutls_hello_ext_set_priv(session,
 					     GNUTLS_EXTENSION_SAFE_RENEGOTIATION,
 					     epriv);
 
@@ -236,7 +238,7 @@ int _gnutls_ext_sr_send_cs(gnutls_session_t session)
 	sr_ext_st *priv;
 	gnutls_ext_priv_data_t epriv;
 
-	ret = _gnutls_ext_get_session_data(session,
+	ret = _gnutls_hello_ext_get_priv(session,
 					   GNUTLS_EXTENSION_SAFE_RENEGOTIATION,
 					   &epriv);
 	if (ret < 0) {
@@ -253,7 +255,7 @@ int _gnutls_ext_sr_send_cs(gnutls_session_t session)
 	}
 
 	if (set != 0)
-		_gnutls_ext_set_session_data(session,
+		_gnutls_hello_ext_set_priv(session,
 					     GNUTLS_EXTENSION_SAFE_RENEGOTIATION,
 					     epriv);
 
@@ -282,7 +284,7 @@ _gnutls_sr_recv_params(gnutls_session_t session,
 		return 0;
 	}
 
-	ret = _gnutls_ext_get_session_data(session,
+	ret = _gnutls_hello_ext_get_priv(session,
 					   GNUTLS_EXTENSION_SAFE_RENEGOTIATION,
 					   &epriv);
 	if (ret < 0
@@ -301,7 +303,7 @@ _gnutls_sr_recv_params(gnutls_session_t session,
 		}
 		epriv = priv;
 
-		_gnutls_ext_set_session_data(session,
+		_gnutls_hello_ext_set_priv(session,
 					     GNUTLS_EXTENSION_SAFE_RENEGOTIATION,
 					     epriv);
 	} else {
@@ -355,7 +357,7 @@ _gnutls_sr_send_params(gnutls_session_t session,
 		return 0;
 	}
 
-	ret = _gnutls_ext_get_session_data(session,
+	ret = _gnutls_hello_ext_get_priv(session,
 					   GNUTLS_EXTENSION_SAFE_RENEGOTIATION,
 					   &epriv);
 	if (ret < 0) {
@@ -370,7 +372,7 @@ _gnutls_sr_send_params(gnutls_session_t session,
 		}
 		epriv = priv;
 
-		_gnutls_ext_set_session_data(session,
+		_gnutls_hello_ext_set_priv(session,
 					     GNUTLS_EXTENSION_SAFE_RENEGOTIATION,
 					     epriv);
 	} else
@@ -434,7 +436,7 @@ unsigned gnutls_safe_renegotiation_status(gnutls_session_t session)
 	sr_ext_st *priv;
 	gnutls_ext_priv_data_t epriv;
 
-	ret = _gnutls_ext_get_session_data(session,
+	ret = _gnutls_hello_ext_get_priv(session,
 					   GNUTLS_EXTENSION_SAFE_RENEGOTIATION,
 					   &epriv);
 	if (ret < 0) {
