@@ -23,15 +23,9 @@
 #ifndef EXT_SERVER_NAME_H
 #define EXT_SERVER_NAME_H
 
-#include <extensions.h>
+#include <hello_ext.h>
 
-typedef struct {
-	uint8_t name[MAX_SERVER_NAME_SIZE+1];
-	unsigned name_length;
-	gnutls_server_name_type_t type;
-} server_name_ext_st;
-
-extern const extension_entry_st ext_mod_server_name;
+extern const hello_ext_entry_st ext_mod_server_name;
 
 unsigned _gnutls_server_name_matches_resumed(gnutls_session_t);
 
