@@ -23,11 +23,18 @@
 #ifndef EXT_SESSION_TICKET_H
 #define EXT_SESSION_TICKET_H
 
-#include <gnutls_extensions.h>
+#include <hello_ext.h>
 
-extern extension_entry_st ext_mod_session_ticket;
+extern const hello_ext_entry_st ext_mod_session_ticket;
 
-int _gnutls_send_new_session_ticket (gnutls_session_t session, int again);
-int _gnutls_recv_new_session_ticket (gnutls_session_t session);
+int _gnutls_send_new_session_ticket(gnutls_session_t session, int again);
+int _gnutls_recv_new_session_ticket(gnutls_session_t session);
+
+int _gnutls_encrypt_session_ticket(gnutls_session_t session,
+				   const gnutls_datum_t *state,
+				   gnutls_datum_t *ticket_data);
+int _gnutls_decrypt_session_ticket(gnutls_session_t session,
+				   const gnutls_datum_t *ticket_data,
+				   gnutls_datum_t *state);
 
 #endif
