@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012 Martin Storsjo
+ * Copyright (C) 2012 Free Software Foundation
  *
  * Author: Martin Storsjo
  *
@@ -7,7 +7,7 @@
  *
  * The GnuTLS is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
+ * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
@@ -22,20 +22,19 @@
 #ifndef EXT_SRTP_H
 #define EXT_SRTP_H
 
-#include <gnutls_extensions.h>
+#include <hello_ext.h>
 
 #define MAX_SRTP_PROFILES 4
 
-typedef struct
-{
-  gnutls_srtp_profile_t profiles[MAX_SRTP_PROFILES];
-  unsigned profiles_size;
-  gnutls_srtp_profile_t selected_profile;
-  uint8_t mki[256];
-  unsigned mki_size;
-  unsigned int mki_received;
+typedef struct {
+	gnutls_srtp_profile_t profiles[MAX_SRTP_PROFILES];
+	unsigned profiles_size;
+	gnutls_srtp_profile_t selected_profile;
+	uint8_t mki[256];
+	unsigned mki_size;
+	unsigned int mki_received;
 } srtp_ext_st;
 
-extern extension_entry_st ext_mod_srtp;
+extern const hello_ext_entry_st ext_mod_srtp;
 
 #endif
