@@ -46,6 +46,11 @@ namespace gnutls
     gnutls_deinit (s);
   }
 
+  gnutls_session_t session::ptr()
+  {
+    return s;
+  }
+
   int session::bye (gnutls_close_request_t how)
   {
     return RETWRAP (gnutls_bye (s, how));
@@ -60,6 +65,11 @@ namespace gnutls
   {
   }
 
+  server_session::server_session (int flags):session (GNUTLS_SERVER |
+			      (flags & ~GNUTLS_CLIENT))
+  {
+  }
+
   server_session::~server_session ()
   {
   }
@@ -277,11 +287,21 @@ namespace gnutls
   {
   }
 
+  client_session::client_session (int flags):session (GNUTLS_CLIENT |
+				  (flags & ~GNUTLS_SERVER))
+  {
+  }
+
   client_session::~client_session ()
   {
   }
 
 // client session
+  void client_session::set_verify_cert (const char *hostname, unsigned flags)
+  {
+    gnutls_session_set_verify_cert(s, hostname, flags);
+  }
+
   void client_session::set_server_name (gnutls_server_name_type_t type,
 					const void *name, size_t name_length)
   {
@@ -459,6 +479,11 @@ namespace gnutls
     gnutls_transport_set_pull_function (s, pull_func);
   }
 
+  void session::set_transport_pull_timeout_function (gnutls_pull_timeout_func pull_timeout_func)
+  {
+    gnutls_transport_set_pull_timeout_function (s, pull_timeout_func);
+  }
+
   void session::set_user_ptr (void *ptr)
   {
     gnutls_session_set_ptr (s, ptr);
