@@ -158,7 +158,7 @@ int _gnutls13_handshake_hash_buffers_synth(gnutls_session_t session,
  * internals, and to security_parameters.
  * this will keep as less data to security_parameters.
  */
-static int resume_copy_required_values(gnutls_session_t session)
+static int tls12_resume_copy_required_vals(gnutls_session_t session, unsigned ticket)
 {
 	int ret;
 
@@ -198,14 +198,18 @@ static int resume_copy_required_values(gnutls_session_t session)
 				    id) < 0)
 		return gnutls_assert_val(GNUTLS_E_UNSUPPORTED_VERSION_PACKET);
 
-	session->security_parameters.cert_type =
-	    session->internals.resumed_security_parameters.cert_type;
+	session->security_parameters.client_ctype =
+	    session->internals.resumed_security_parameters.client_ctype;
+	session->security_parameters.server_ctype =
+	    session->internals.resumed_security_parameters.server_ctype;
 
-	memcpy(session->security_parameters.session_id,
-	       session->internals.resumed_security_parameters.session_id,
-	       sizeof(session->security_parameters.session_id));
-	session->security_parameters.session_id_size =
-	    session->internals.resumed_security_parameters.session_id_size;
+	if (!ticket) {
+		memcpy(session->security_parameters.session_id,
+		       session->internals.resumed_security_parameters.session_id,
+		       sizeof(session->security_parameters.session_id));
+		session->security_parameters.session_id_size =
+		    session->internals.resumed_security_parameters.session_id_size;
+	}
 
 	return 0;
 }
@@ -427,21 +431,21 @@ _gnutls_finished(gnutls_session_t session, int type, void *ret,
  */
 int
 _gnutls_negotiate_version(gnutls_session_t session,
-			  gnutls_protocol_t adv_version, uint8_t major, uint8_t minor)
+			  uint8_t major, uint8_t minor, unsigned allow_tls13)
 {
 	const version_entry_st *vers;
+	const version_entry_st *aversion = nversion_to_entry(major, minor);
 
 	/* if we do not support that version, unless that version is TLS 1.2;
 	 * TLS 1.2 is handled separately because it is always advertized under TLS 1.3 or later */
-	if (adv_version == GNUTLS_VERSION_UNKNOWN ||
-	    _gnutls_version_is_supported(session, adv_version) == 0) {
+	if (aversion == NULL ||
+	    _gnutls_nversion_is_supported(session, major, minor) == 0) {
 
-		if (adv_version == GNUTLS_TLS1_2) {
+		if (aversion && aversion->id == GNUTLS_TLS1_2) {
 			vers = _gnutls_version_max(session);
 			if (vers->id >= GNUTLS_TLS1_2) {
-				if (_gnutls_set_current_version(session, adv_version) < 0)
-					return gnutls_assert_val(GNUTLS_E_UNSUPPORTED_VERSION_PACKET);
-				return adv_version;
+				session->security_parameters.pversion = aversion;
+				return 0;
 			}
 		}
 
@@ -463,12 +467,17 @@ _gnutls_negotiate_version(gnutls_session_t session,
 
 		session->security_parameters.pversion = vers;
 
-		return vers->id;
+		return 0;
 	} else {
-		if (_gnutls_set_current_version(session, adv_version) < 0)
-			return gnutls_assert_val(GNUTLS_E_UNSUPPORTED_VERSION_PACKET);
+		session->security_parameters.pversion = aversion;
 
-		return adv_version;
+		/* we do not allow TLS1.3 negotiation using this mechanism */
+		if (aversion->tls13_sem && !allow_tls13) {
+			vers = _gnutls_legacy_version_max(session);
+			session->security_parameters.pversion = vers;
+		}
+
+		return 0;
 	}
 }
 
@@ -479,7 +488,7 @@ _gnutls_negotiate_version(gnutls_session_t session,
  */
 int
 _gnutls_user_hello_func(gnutls_session_t session,
-			gnutls_protocol_t adv_version, uint8_t major, uint8_t minor)
+			uint8_t major, uint8_t minor)
 {
 	int ret, sret = 0;
 	const version_entry_st *vers;
@@ -502,7 +511,7 @@ _gnutls_user_hello_func(gnutls_session_t session,
 			 * earlier, as TLS1.3 uses a different set of ciphersuites, and
 			 * thus we cannot fallback.
 			 */
-			ret = _gnutls_negotiate_version(session, adv_version, major, minor);
+			ret = _gnutls_negotiate_version(session, major, minor, 0);
 			if (ret < 0) {
 				gnutls_assert();
 				return ret;
@@ -547,7 +556,6 @@ read_client_hello(gnutls_session_t session, uint8_t * data,
 	int pos = 0, ret;
 	uint16_t suite_size, comp_size;
 	int ext_size;
-	gnutls_protocol_t adv_version;
 	int neg_version, sret = 0;
 	int len = datalen;
 	uint8_t major, minor;
@@ -558,17 +566,20 @@ read_client_hello(gnutls_session_t session, uint8_t * data,
 	_gnutls_handshake_log("HSK[%p]: Client's version: %d.%d\n",
 			      session, data[pos], data[pos + 1]);
 
-	adv_version = _gnutls_version_get(data[pos], data[pos + 1]);
-
 	major = data[pos];
 	minor = data[pos+1];
+
 	set_adv_version(session, major, minor);
 
-	neg_version = _gnutls_negotiate_version(session, adv_version, major, minor);
-	if (neg_version < 0) {
-		gnutls_assert();
-		return neg_version;
-	}
+	ret = _gnutls_negotiate_version(session, major, minor, 0);
+	if (ret < 0)
+		return gnutls_assert_val(ret);
+
+	vers = get_version(session);
+	if (vers == NULL)
+		return gnutls_assert_val(GNUTLS_E_UNSUPPORTED_VERSION_PACKET);
+
+	neg_version = vers->id;
 
 	pos += 2;
 
@@ -667,13 +678,13 @@ read_client_hello(gnutls_session_t session, uint8_t * data,
 			if (ret < 0)
 				return gnutls_assert_val(ret);
 
-			ret = resume_copy_required_values(session);
+			ret = tls12_resume_copy_required_vals(session, 0);
 			if (ret < 0)
 				return gnutls_assert_val(ret);
 
 			session->internals.resumed = RESUME_TRUE;
 
-			return _gnutls_user_hello_func(session, adv_version, major, minor);
+			return _gnutls_user_hello_func(session, major, minor);
 		} else {
 			ret = _gnutls_generate_session_id(session->security_parameters.
 							  session_id,
@@ -707,7 +718,7 @@ read_client_hello(gnutls_session_t session, uint8_t * data,
 	}
 
 	/* we cache this error code */
-	sret = _gnutls_user_hello_func(session, adv_version, major, minor);
+	sret = _gnutls_user_hello_func(session, major, minor);
 	if (sret < 0 && sret != GNUTLS_E_INT_RET_0) {
 		gnutls_assert();
 		return sret;
@@ -724,12 +735,6 @@ read_client_hello(gnutls_session_t session, uint8_t * data,
 
 	/* resumed by session_ticket extension */
 	if (!vers->tls13_sem && session->internals.resumed != RESUME_FALSE) {
-		/* to indicate the client that the current session is resumed */
-		memcpy(session->internals.resumed_security_parameters.
-		       session_id, session_id, session_id_len);
-		session->internals.resumed_security_parameters.
-		    session_id_size = session_id_len;
-
 		session->internals.resumed_security_parameters.
 		    max_record_recv_size =
 		    session->security_parameters.max_record_recv_size;
@@ -741,10 +746,14 @@ read_client_hello(gnutls_session_t session, uint8_t * data,
 		if (ret < 0)
 			return gnutls_assert_val(ret);
 
-		ret = resume_copy_required_values(session);
+		ret = tls12_resume_copy_required_vals(session, 1);
 		if (ret < 0)
 			return gnutls_assert_val(ret);
 
+		/* to indicate to the client that the current session is resumed */
+		memcpy(session->security_parameters.session_id, session_id, session_id_len);
+		session->security_parameters.session_id_size = session_id_len;
+
 		return 0;
 	}
 
@@ -1822,7 +1831,7 @@ read_server_hello(gnutls_session_t session,
 			return gnutls_assert_val(GNUTLS_E_UNSUPPORTED_VERSION_PACKET);
 	}
 
-	if (_gnutls_version_is_supported(session, vers->id) == 0)
+	if (_gnutls_nversion_is_supported(session, vers->major, vers->minor) == 0)
 		return gnutls_assert_val(GNUTLS_E_UNSUPPORTED_VERSION_PACKET);
 
 	/* set server random - done after final version is selected */
@@ -1991,12 +2000,13 @@ static int send_client_hello(gnutls_session_t session, int again)
 	uint8_t session_id_len =
 	    session->internals.resumed_security_parameters.session_id_size;
 
-	/* note that rehandshake is different than resuming
-	 */
-	if (session->security_parameters.session_id_size)
-		rehandshake = 1;
 
 	if (again == 0) {
+		/* note that rehandshake is different than resuming
+		 */
+		if (session->internals.initial_negotiation_completed)
+			rehandshake = 1;
+
 		ret = _gnutls_buffer_init_handshake_mbuffer(&extdata);
 		if (ret < 0)
 			return gnutls_assert_val(ret);
@@ -2049,13 +2059,15 @@ static int send_client_hello(gnutls_session_t session, int again)
 			goto cleanup;
 		}
 
-		/* Set the version we advertized as maximum 
-		 * (RSA uses it).
-		 */
-		set_adv_version(session, hver->major, hver->minor);
-		if (_gnutls_set_current_version(session, hver->id) < 0) {
-			ret = gnutls_assert_val(GNUTLS_E_UNSUPPORTED_VERSION_PACKET);
-			goto cleanup;
+		/* if we are replying to an HRR the version is already negotiated */
+		if (!(session->internals.hsk_flags & HSK_HRR_RECEIVED) || !get_version(session)) {
+			/* Set the version we advertized as maximum
+			 * (RSA uses it). */
+			set_adv_version(session, hver->major, hver->minor);
+			if (_gnutls_set_current_version(session, hver->id) < 0) {
+				ret = gnutls_assert_val(GNUTLS_E_UNSUPPORTED_VERSION_PACKET);
+				goto cleanup;
+			}
 		}
 
 		if (session->internals.priorities->min_record_version != 0) {
@@ -2305,6 +2317,18 @@ int _gnutls_send_server_hello(gnutls_session_t session, int again)
 				gnutls_assert();
 				goto fail;
 			}
+
+			/* Under TLS1.3, the session ID is used for different purposes than
+			 * the TLS1.0 session ID. Ensure that there is an internally set
+			 * value which the server will see on the original and resumed sessions */
+			ret = _gnutls_generate_session_id(session->security_parameters.
+							  session_id,
+							  &session->security_parameters.
+							  session_id_size);
+			if (ret < 0) {
+				gnutls_assert();
+				goto fail;
+			}
 		}
 
 		bufel = _gnutls_buffer_to_mbuffer(&buf);
@@ -2460,8 +2484,9 @@ int gnutls_rehandshake(gnutls_session_t session)
 	if (session->security_parameters.entity == GNUTLS_CLIENT)
 		return GNUTLS_E_INVALID_REQUEST;
 
-	if (vers->tls13_sem)
+	if (vers->tls13_sem) {
 		return gnutls_session_key_update(session, GNUTLS_KU_PEER);
+	}
 
 	_dtls_async_timer_delete(session);
 
@@ -2650,9 +2675,8 @@ int gnutls_handshake(gnutls_session_t session)
 	}
 
 	/* clear handshake buffer */
-	if (session->security_parameters.entity != GNUTLS_CLIENT ||
-	    !(session->internals.flags & GNUTLS_ENABLE_FALSE_START) ||
-	    session->internals.recv_state != RECV_STATE_FALSE_START) {
+	if (session->internals.recv_state != RECV_STATE_FALSE_START &&
+	    session->internals.recv_state != RECV_STATE_EARLY_START) {
 
 		_gnutls_handshake_hash_buffers_clear(session);
 
@@ -2791,7 +2815,7 @@ static int handshake_client(gnutls_session_t session)
 		ret = send_client_hello(session, AGAIN(STATE1));
 		STATE = STATE1;
 		IMED_RET("send hello", ret, 1);
-		/* fall through */
+		FALLTHROUGH;
 	case STATE2:
 		if (IS_DTLS(session)) {
 			ret =
@@ -2818,7 +2842,7 @@ static int handshake_client(gnutls_session_t session)
 				return 1;
 			}
 		}
-		/* fall through */
+		FALLTHROUGH;
 	case STATE3:
 		/* receive the server hello */
 		ret =
@@ -2827,7 +2851,7 @@ static int handshake_client(gnutls_session_t session)
 					   0, NULL);
 		STATE = STATE3;
 		IMED_RET("recv hello", ret, 1);
-		/* fall through */
+		FALLTHROUGH;
 	case STATE4:
 		ver = get_version(session);
 		if (ver->tls13_sem) { /* TLS 1.3 state machine */
@@ -2837,22 +2861,22 @@ static int handshake_client(gnutls_session_t session)
 
 		ret = _gnutls_ext_sr_verify(session);
 		STATE = STATE4;
-		IMED_RET("recv hello", ret, 0);
-		/* fall through */
+		IMED_RET_FATAL("recv hello", ret, 0);
+		FALLTHROUGH;
 	case STATE5:
 		if (session->security_parameters.do_recv_supplemental) {
 			ret = _gnutls_recv_supplemental(session);
 			STATE = STATE5;
 			IMED_RET("recv supplemental", ret, 1);
 		}
-		/* fall through */
+		FALLTHROUGH;
 	case STATE6:
 		/* RECV CERTIFICATE */
 		if (session->internals.resumed == RESUME_FALSE)	/* if we are not resuming */
 			ret = _gnutls_recv_server_certificate(session);
 		STATE = STATE6;
 		IMED_RET("recv server certificate", ret, 1);
-		/* fall through */
+		FALLTHROUGH;
 	case STATE7:
 #ifdef ENABLE_OCSP
 		/* RECV CERTIFICATE STATUS */
@@ -2863,7 +2887,7 @@ static int handshake_client(gnutls_session_t session)
 		STATE = STATE7;
 		IMED_RET("recv server certificate", ret, 1);
 #endif
-		/* fall through */
+		FALLTHROUGH;
 	case STATE8:
 		ret = _gnutls_run_verify_callback(session, GNUTLS_CLIENT);
 		STATE = STATE8;
@@ -2877,7 +2901,7 @@ static int handshake_client(gnutls_session_t session)
 			ret = _gnutls_recv_server_kx_message(session);
 		STATE = STATE9;
 		IMED_RET("recv server kx message", ret, 1);
-		/* fall through */
+		FALLTHROUGH;
 	case STATE10:
 		/* receive the server certificate request - if any 
 		 */
@@ -2887,7 +2911,7 @@ static int handshake_client(gnutls_session_t session)
 		STATE = STATE10;
 		IMED_RET("recv server certificate request message", ret,
 			 1);
-		/* fall through */
+		FALLTHROUGH;
 	case STATE11:
 		/* receive the server hello done */
 		if (session->internals.resumed == RESUME_FALSE)	/* if we are not resuming */
@@ -2897,7 +2921,7 @@ static int handshake_client(gnutls_session_t session)
 						   0, NULL);
 		STATE = STATE11;
 		IMED_RET("recv server hello done", ret, 1);
-		/* fall through */
+		FALLTHROUGH;
 	case STATE12:
 		if (session->security_parameters.do_send_supplemental) {
 			ret =
@@ -2906,7 +2930,7 @@ static int handshake_client(gnutls_session_t session)
 			STATE = STATE12;
 			IMED_RET("send supplemental", ret, 0);
 		}
-		/* fall through */
+		FALLTHROUGH;
 	case STATE13:
 		/* send our certificate - if any and if requested
 		 */
@@ -2917,7 +2941,7 @@ static int handshake_client(gnutls_session_t session)
 							    (STATE13));
 		STATE = STATE13;
 		IMED_RET("send client certificate", ret, 0);
-		/* fall through */
+		FALLTHROUGH;
 	case STATE14:
 		if (session->internals.resumed == RESUME_FALSE)	/* if we are not resuming */
 			ret =
@@ -2925,7 +2949,7 @@ static int handshake_client(gnutls_session_t session)
 							   AGAIN(STATE14));
 		STATE = STATE14;
 		IMED_RET("send client kx", ret, 0);
-		/* fall through */
+		FALLTHROUGH;
 	case STATE15:
 		/* send client certificate verify */
 		if (session->internals.resumed == RESUME_FALSE)	/* if we are not resuming */
@@ -2935,7 +2959,7 @@ static int handshake_client(gnutls_session_t session)
 								   (STATE15));
 		STATE = STATE15;
 		IMED_RET("send client certificate verify", ret, 1);
-		/* fall through */
+		FALLTHROUGH;
 	case STATE16:
 		STATE = STATE16;
 		if (session->internals.resumed == RESUME_FALSE) {
@@ -2946,7 +2970,7 @@ static int handshake_client(gnutls_session_t session)
 			IMED_RET("recv handshake new session ticket", ret,
 				 1);
 		}
-		/* fall through */
+		FALLTHROUGH;
 	case STATE17:
 		STATE = STATE17;
 		if (session->internals.resumed == RESUME_FALSE && (session->internals.flags & GNUTLS_ENABLE_FALSE_START) && can_send_false_start(session)) {
@@ -2960,7 +2984,7 @@ static int handshake_client(gnutls_session_t session)
 
 			return 0;
 		}
-		/* fall through */
+		FALLTHROUGH;
 	case STATE18:
 		STATE = STATE18;
 
@@ -2972,7 +2996,7 @@ static int handshake_client(gnutls_session_t session)
 			ret = recv_handshake_final(session, TRUE);
 			IMED_RET("recv handshake final", ret, 1);
 		}
-		/* fall through */
+		FALLTHROUGH;
 	case STATE19:
 		STATE = STATE19;
 		if (session->internals.resumed == RESUME_FALSE) {
@@ -2984,7 +3008,7 @@ static int handshake_client(gnutls_session_t session)
 		}
 
 		STATE = STATE0;
-		/* fall through */
+		FALLTHROUGH;
 	default:
 		break;
 	}
@@ -3055,7 +3079,7 @@ ssize_t _gnutls_send_change_cipher_spec(gnutls_session_t session, int again)
 
 		/* under TLS 1.3, CCS may be immediately followed by
 		 * receiving ClientHello thus cannot be cached */
-		if (vers && vers->tls13_sem) {
+		if (vers->tls13_sem) {
 			ret = _gnutls_handshake_io_write_flush(session);
 			if (ret < 0)
 				return gnutls_assert_val(ret);
@@ -3102,7 +3126,7 @@ static int send_handshake_final(gnutls_session_t session, int init)
 			return ret;
 		}
 
-		/* fall through */
+		FALLTHROUGH;
 	case STATE2:
 		/* send the finished message */
 		ret = _gnutls_send_finished(session, FAGAIN(STATE2));
@@ -3113,7 +3137,7 @@ static int send_handshake_final(gnutls_session_t session, int init)
 		}
 
 		FINAL_STATE = STATE0;
-		/* fall through */
+		FALLTHROUGH;
 	default:
 		break;
 	}
@@ -3186,7 +3210,7 @@ static int recv_handshake_final(gnutls_session_t session, int init)
 			gnutls_assert();
 			return ret;
 		}
-		/* fall through */
+		FALLTHROUGH;
 	case STATE31:
 		FINAL_STATE = STATE31;
 
@@ -3204,7 +3228,7 @@ static int recv_handshake_final(gnutls_session_t session, int init)
 			return ret;
 		}
 		FINAL_STATE = STATE0;
-		/* fall through */
+		FALLTHROUGH;
 	default:
 		break;
 	}
@@ -3250,13 +3274,13 @@ static int handshake_server(gnutls_session_t session)
 		}
 
 		IMED_RET("recv hello", ret, 1);
-		/* fall through */
+		FALLTHROUGH;
 	case STATE2:
 
 		ret = _gnutls_ext_sr_verify(session);
 		STATE = STATE2;
-		IMED_RET("recv hello", ret, 0);
-		/* fall through */
+		IMED_RET_FATAL("recv hello", ret, 0);
+		FALLTHROUGH;
 	case STATE3:
 		ret = _gnutls_send_server_hello(session, AGAIN(STATE3));
 		STATE = STATE3;
@@ -3268,7 +3292,7 @@ static int handshake_server(gnutls_session_t session)
 			goto reset;
 		}
 
-		/* fall through */
+		FALLTHROUGH;
 	case STATE4:
 		if (session->security_parameters.do_send_supplemental) {
 			ret =
@@ -3278,7 +3302,7 @@ static int handshake_server(gnutls_session_t session)
 			IMED_RET("send supplemental data", ret, 0);
 		}
 		/* SEND CERTIFICATE + KEYEXCHANGE + CERTIFICATE_REQUEST */
-		/* fall through */
+		FALLTHROUGH;
 	case STATE5:
 		/* NOTE: these should not be send if we are resuming */
 
@@ -3288,7 +3312,7 @@ static int handshake_server(gnutls_session_t session)
 							    AGAIN(STATE5));
 		STATE = STATE5;
 		IMED_RET("send server certificate", ret, 0);
-		/* fall through */
+		FALLTHROUGH;
 	case STATE6:
 #ifdef ENABLE_OCSP
 		if (session->internals.resumed == RESUME_FALSE)
@@ -3299,7 +3323,7 @@ static int handshake_server(gnutls_session_t session)
 		STATE = STATE6;
 		IMED_RET("send server certificate status", ret, 0);
 #endif
-		/* fall through */
+		FALLTHROUGH;
 	case STATE7:
 		/* send server key exchange (A) */
 		if (session->internals.resumed == RESUME_FALSE)
@@ -3308,7 +3332,7 @@ static int handshake_server(gnutls_session_t session)
 							   AGAIN(STATE7));
 		STATE = STATE7;
 		IMED_RET("send server kx", ret, 0);
-		/* fall through */
+		FALLTHROUGH;
 	case STATE8:
 		/* Send certificate request - if requested to */
 		if (session->internals.resumed == RESUME_FALSE)
@@ -3317,7 +3341,7 @@ static int handshake_server(gnutls_session_t session)
 							    AGAIN(STATE8));
 		STATE = STATE8;
 		IMED_RET("send server cert request", ret, 0);
-		/* fall through */
+		FALLTHROUGH;
 	case STATE9:
 		/* send the server hello done */
 		if (session->internals.resumed == RESUME_FALSE)	/* if we are not resuming */
@@ -3327,7 +3351,7 @@ static int handshake_server(gnutls_session_t session)
 							 AGAIN(STATE9));
 		STATE = STATE9;
 		IMED_RET("send server hello done", ret, 1);
-		/* fall through */
+		FALLTHROUGH;
 	case STATE10:
 		if (session->security_parameters.do_recv_supplemental) {
 			ret = _gnutls_recv_supplemental(session);
@@ -3335,27 +3359,27 @@ static int handshake_server(gnutls_session_t session)
 			IMED_RET("recv client supplemental", ret, 1);
 		}
 		/* RECV CERTIFICATE + KEYEXCHANGE + CERTIFICATE_VERIFY */
-		/* fall through */
+		FALLTHROUGH;
 	case STATE11:
 		/* receive the client certificate message */
 		if (session->internals.resumed == RESUME_FALSE)	/* if we are not resuming */
 			ret = _gnutls_recv_client_certificate(session);
 		STATE = STATE11;
 		IMED_RET("recv client certificate", ret, 1);
-		/* fall through */
+		FALLTHROUGH;
 	case STATE12:
 		ret = _gnutls_run_verify_callback(session, GNUTLS_SERVER);
 		STATE = STATE12;
 		if (ret < 0)
 			return gnutls_assert_val(ret);
-		/* fall through */
+		FALLTHROUGH;
 	case STATE13:
 		/* receive the client key exchange message */
 		if (session->internals.resumed == RESUME_FALSE)	/* if we are not resuming */
 			ret = _gnutls_recv_client_kx_message(session);
 		STATE = STATE13;
 		IMED_RET("recv client kx", ret, 1);
-		/* fall through */
+		FALLTHROUGH;
 	case STATE14:
 		/* receive the client certificate verify message */
 		if (session->internals.resumed == RESUME_FALSE)	/* if we are not resuming */
@@ -3364,7 +3388,7 @@ static int handshake_server(gnutls_session_t session)
 			    (session);
 		STATE = STATE14;
 		IMED_RET("recv client certificate verify", ret, 1);
-		/* fall through */
+		FALLTHROUGH;
 	case STATE15:
 		STATE = STATE15;
 		if (session->internals.resumed == RESUME_FALSE) {	/* if we are not resuming */
@@ -3374,14 +3398,14 @@ static int handshake_server(gnutls_session_t session)
 			ret = send_handshake_final(session, TRUE);
 			IMED_RET("send handshake final 2", ret, 1);
 		}
-		/* fall through */
+		FALLTHROUGH;
 	case STATE16:
 		ret =
 		    _gnutls_send_new_session_ticket(session,
 						    AGAIN(STATE16));
 		STATE = STATE16;
 		IMED_RET("send handshake new session ticket", ret, 0);
-		/* fall through */
+		FALLTHROUGH;
 	case STATE17:
 		STATE = STATE17;
 		if (session->internals.resumed == RESUME_FALSE) {	/* if we are not resuming */
@@ -3401,7 +3425,7 @@ static int handshake_server(gnutls_session_t session)
 		}
 
 		STATE = STATE0;
-		/* fall through */
+		FALLTHROUGH;
 	default:
 		break;
 	}
@@ -3409,15 +3433,15 @@ static int handshake_server(gnutls_session_t session)
 	return _gnutls_check_id_for_change(session);
 }
 
-int _gnutls_generate_session_id(uint8_t * session_id, uint8_t * len)
+int _gnutls_generate_session_id(uint8_t * session_id, uint8_t *len)
 {
 	int ret;
 
-	*len = GNUTLS_MAX_SESSION_ID_SIZE;
+	*len = GNUTLS_DEF_SESSION_ID_SIZE;
 
 	ret =
 	    gnutls_rnd(GNUTLS_RND_NONCE, session_id,
-			GNUTLS_MAX_SESSION_ID_SIZE);
+			GNUTLS_DEF_SESSION_ID_SIZE);
 	if (ret < 0) {
 		gnutls_assert();
 		return ret;
