@@ -33,6 +33,11 @@
 #include <nettle/arctwo.h>
 #include <nettle/salsa20.h>
 #include <nettle/des.h>
+#include <nettle/version.h>
+#if ENABLE_GOST
+#include "gost/gost28147.h"
+#include <nettle/cfb.h>
+#endif
 #include <nettle/nettle-meta.h>
 #include <nettle/cbc.h>
 #include <nettle/gcm.h>
@@ -87,7 +92,6 @@ struct nettle_cipher_st {
 	nettle_set_key_func* set_decrypt_key;
 	gen_setkey_func gen_set_key; /* for arcfour which has variable key size */
 	setiv_func set_iv;
-	unsigned fips_allowed;
 };
 
 struct nettle_cipher_ctx {
@@ -131,6 +135,61 @@ _cbc_decrypt(struct nettle_cipher_ctx *ctx, size_t length, uint8_t * dst,
 		    length, dst, src);
 }
 
+#if ENABLE_GOST
+static void
+_cfb_encrypt(struct nettle_cipher_ctx *ctx, size_t length, uint8_t * dst,
+		const uint8_t * src)
+{
+	cfb_encrypt(ctx->ctx_ptr, ctx->cipher->encrypt_block,
+		    ctx->iv_size, ctx->iv,
+		    length, dst, src);
+}
+
+static void
+_cfb_decrypt(struct nettle_cipher_ctx *ctx, size_t length, uint8_t * dst,
+		const uint8_t * src)
+{
+	cfb_decrypt(ctx->ctx_ptr, ctx->cipher->encrypt_block,
+		    ctx->iv_size, ctx->iv,
+		    length, dst, src);
+}
+
+static void
+_gost28147_set_key_tc26z(void *ctx, const uint8_t *key)
+{
+	gost28147_set_key(ctx, key);
+	gost28147_set_param(ctx, &gost28147_param_TC26_Z);
+}
+
+static void
+_gost28147_set_key_cpa(void *ctx, const uint8_t *key)
+{
+	gost28147_set_key(ctx, key);
+	gost28147_set_param(ctx, &gost28147_param_CryptoPro_A);
+}
+
+static void
+_gost28147_set_key_cpb(void *ctx, const uint8_t *key)
+{
+	gost28147_set_key(ctx, key);
+	gost28147_set_param(ctx, &gost28147_param_CryptoPro_A);
+}
+
+static void
+_gost28147_set_key_cpc(void *ctx, const uint8_t *key)
+{
+	gost28147_set_key(ctx, key);
+	gost28147_set_param(ctx, &gost28147_param_CryptoPro_A);
+}
+
+static void
+_gost28147_set_key_cpd(void *ctx, const uint8_t *key)
+{
+	gost28147_set_key(ctx, key);
+	gost28147_set_param(ctx, &gost28147_param_CryptoPro_A);
+}
+#endif
+
 static void
 _ccm_encrypt(struct nettle_cipher_ctx *ctx,
 	      size_t nonce_size, const void *nonce,
@@ -189,6 +248,16 @@ _gcm_decrypt(struct nettle_cipher_ctx *ctx, size_t length, uint8_t * dst,
 		    length, dst, src);
 }
 
+static void _des_set_key(struct des_ctx *ctx, const uint8_t *key)
+{
+	des_set_key(ctx, key);
+}
+
+static void _des3_set_key(struct des3_ctx *ctx, const uint8_t *key)
+{
+	des3_set_key(ctx, key);
+}
+
 static const struct nettle_cipher_st builtin_ciphers[] = {
 	{  .algo = GNUTLS_CIPHER_AES_128_GCM,
 	   .block_size = AES_BLOCK_SIZE,
@@ -206,7 +275,6 @@ static const struct nettle_cipher_st builtin_ciphers[] = {
 	   .auth = (nettle_hash_update_func*)gcm_aes128_update,
 	   .set_iv = (setiv_func)gcm_aes128_set_iv,
 	   .max_iv_size = GCM_IV_SIZE,
-	   .fips_allowed = 1
 	},
 	{  .algo = GNUTLS_CIPHER_AES_256_GCM,
 	   .block_size = AES_BLOCK_SIZE,
@@ -224,7 +292,6 @@ static const struct nettle_cipher_st builtin_ciphers[] = {
 	   .auth = (nettle_hash_update_func*)gcm_aes256_update,
 	   .set_iv = (setiv_func)gcm_aes256_set_iv,
 	   .max_iv_size = GCM_IV_SIZE,
-	   .fips_allowed = 1
 	},
 	{  .algo = GNUTLS_CIPHER_AES_128_CCM,
 	   .block_size = AES_BLOCK_SIZE,
@@ -238,7 +305,6 @@ static const struct nettle_cipher_st builtin_ciphers[] = {
 	   .set_encrypt_key = (nettle_set_key_func*)aes128_set_encrypt_key,
 	   .set_decrypt_key = (nettle_set_key_func*)aes128_set_encrypt_key,
 	   .max_iv_size = CCM_MAX_NONCE_SIZE,
-	   .fips_allowed = 1
 	},
 	{  .algo = GNUTLS_CIPHER_AES_128_CCM_8,
 	   .block_size = AES_BLOCK_SIZE,
@@ -252,7 +318,6 @@ static const struct nettle_cipher_st builtin_ciphers[] = {
 	   .set_encrypt_key = (nettle_set_key_func*)aes128_set_encrypt_key,
 	   .set_decrypt_key = (nettle_set_key_func*)aes128_set_encrypt_key,
 	   .max_iv_size = CCM_MAX_NONCE_SIZE,
-	   .fips_allowed = 1
 	},
 	{  .algo = GNUTLS_CIPHER_AES_256_CCM,
 	   .block_size = AES_BLOCK_SIZE,
@@ -266,7 +331,6 @@ static const struct nettle_cipher_st builtin_ciphers[] = {
 	   .set_encrypt_key = (nettle_set_key_func*)aes256_set_encrypt_key,
 	   .set_decrypt_key = (nettle_set_key_func*)aes256_set_encrypt_key,
 	   .max_iv_size = CCM_MAX_NONCE_SIZE,
-	   .fips_allowed = 1
 	},
 	{  .algo = GNUTLS_CIPHER_AES_256_CCM_8,
 	   .block_size = AES_BLOCK_SIZE,
@@ -280,7 +344,6 @@ static const struct nettle_cipher_st builtin_ciphers[] = {
 	   .set_encrypt_key = (nettle_set_key_func*)aes256_set_encrypt_key,
 	   .set_decrypt_key = (nettle_set_key_func*)aes256_set_encrypt_key,
 	   .max_iv_size = CCM_MAX_NONCE_SIZE,
-	   .fips_allowed = 1
 	},
 	{  .algo = GNUTLS_CIPHER_CAMELLIA_128_GCM,
 	   .block_size = CAMELLIA_BLOCK_SIZE,
@@ -326,7 +389,6 @@ static const struct nettle_cipher_st builtin_ciphers[] = {
 	   .set_encrypt_key = (nettle_set_key_func*)aes128_set_encrypt_key,
 	   .set_decrypt_key = (nettle_set_key_func*)aes128_set_decrypt_key,
 	   .max_iv_size = AES_BLOCK_SIZE,
-	   .fips_allowed = 1
 	},
 	{  .algo = GNUTLS_CIPHER_AES_192_CBC,
 	   .block_size = AES_BLOCK_SIZE,
@@ -340,7 +402,6 @@ static const struct nettle_cipher_st builtin_ciphers[] = {
 	   .set_encrypt_key = (nettle_set_key_func*)aes192_set_encrypt_key,
 	   .set_decrypt_key = (nettle_set_key_func*)aes192_set_decrypt_key,
 	   .max_iv_size = AES_BLOCK_SIZE,
-	   .fips_allowed = 1
 	},
 	{  .algo = GNUTLS_CIPHER_AES_256_CBC,
 	   .block_size = AES_BLOCK_SIZE,
@@ -354,7 +415,6 @@ static const struct nettle_cipher_st builtin_ciphers[] = {
 	   .set_encrypt_key = (nettle_set_key_func*)aes256_set_encrypt_key,
 	   .set_decrypt_key = (nettle_set_key_func*)aes256_set_decrypt_key,
 	   .max_iv_size = AES_BLOCK_SIZE,
-	   .fips_allowed = 1
 	},
 	{  .algo = GNUTLS_CIPHER_CAMELLIA_128_CBC,
 	   .block_size = CAMELLIA_BLOCK_SIZE,
@@ -417,8 +477,8 @@ static const struct nettle_cipher_st builtin_ciphers[] = {
 	   .ctx_size = sizeof(struct CBC_CTX(struct des_ctx, DES_BLOCK_SIZE)),
 	   .encrypt = _cbc_encrypt,
 	   .decrypt = _cbc_decrypt,
-	   .set_encrypt_key = (nettle_set_key_func*)des_set_key,
-	   .set_decrypt_key = (nettle_set_key_func*)des_set_key,
+	   .set_encrypt_key = (nettle_set_key_func*)_des_set_key,
+	   .set_decrypt_key = (nettle_set_key_func*)_des_set_key,
 	   .max_iv_size = DES_BLOCK_SIZE,
 	},
 	{  .algo = GNUTLS_CIPHER_3DES_CBC,
@@ -430,10 +490,9 @@ static const struct nettle_cipher_st builtin_ciphers[] = {
 	   .ctx_size = sizeof(struct CBC_CTX(struct des3_ctx, DES3_BLOCK_SIZE)),
 	   .encrypt = _cbc_encrypt,
 	   .decrypt = _cbc_decrypt,
-	   .set_encrypt_key = (nettle_set_key_func*)des3_set_key,
-	   .set_decrypt_key = (nettle_set_key_func*)des3_set_key,
+	   .set_encrypt_key = (nettle_set_key_func*)_des3_set_key,
+	   .set_decrypt_key = (nettle_set_key_func*)_des3_set_key,
 	   .max_iv_size = DES_BLOCK_SIZE,
-	   .fips_allowed = 1
 	},
 	{  .algo = GNUTLS_CIPHER_ARCFOUR_128,
 	   .block_size = 1,
@@ -490,6 +549,73 @@ static const struct nettle_cipher_st builtin_ciphers[] = {
 	   .set_iv = (setiv_func)_chacha_poly1305_set_nonce,
 	   .max_iv_size = CHACHA_POLY1305_NONCE_SIZE,
 	},
+#if ENABLE_GOST
+	{
+	   .algo = GNUTLS_CIPHER_GOST28147_TC26Z_CFB,
+	   .block_size = GOST28147_BLOCK_SIZE,
+	   .key_size = GOST28147_KEY_SIZE,
+	   .encrypt_block = (nettle_cipher_func*)gost28147_encrypt_for_cfb,
+	   .decrypt_block = (nettle_cipher_func*)gost28147_encrypt_for_cfb,
+
+	   .ctx_size = sizeof(struct CFB_CTX(struct gost28147_ctx, GOST28147_BLOCK_SIZE)),
+	   .encrypt = _cfb_encrypt,
+	   .decrypt = _cfb_decrypt,
+	   .set_encrypt_key = _gost28147_set_key_tc26z,
+	   .set_decrypt_key = _gost28147_set_key_tc26z,
+	},
+	{
+	   .algo = GNUTLS_CIPHER_GOST28147_CPA_CFB,
+	   .block_size = GOST28147_BLOCK_SIZE,
+	   .key_size = GOST28147_KEY_SIZE,
+	   .encrypt_block = (nettle_cipher_func*)gost28147_encrypt_for_cfb,
+	   .decrypt_block = (nettle_cipher_func*)gost28147_encrypt_for_cfb,
+
+	   .ctx_size = sizeof(struct CFB_CTX(struct gost28147_ctx, GOST28147_BLOCK_SIZE)),
+	   .encrypt = _cfb_encrypt,
+	   .decrypt = _cfb_decrypt,
+	   .set_encrypt_key = _gost28147_set_key_cpa,
+	   .set_decrypt_key = _gost28147_set_key_cpa,
+	},
+	{
+	   .algo = GNUTLS_CIPHER_GOST28147_CPB_CFB,
+	   .block_size = GOST28147_BLOCK_SIZE,
+	   .key_size = GOST28147_KEY_SIZE,
+	   .encrypt_block = (nettle_cipher_func*)gost28147_encrypt_for_cfb,
+	   .decrypt_block = (nettle_cipher_func*)gost28147_encrypt_for_cfb,
+
+	   .ctx_size = sizeof(struct CFB_CTX(struct gost28147_ctx, GOST28147_BLOCK_SIZE)),
+	   .encrypt = _cfb_encrypt,
+	   .decrypt = _cfb_decrypt,
+	   .set_encrypt_key = _gost28147_set_key_cpb,
+	   .set_decrypt_key = _gost28147_set_key_cpb,
+	},
+	{
+	   .algo = GNUTLS_CIPHER_GOST28147_CPC_CFB,
+	   .block_size = GOST28147_BLOCK_SIZE,
+	   .key_size = GOST28147_KEY_SIZE,
+	   .encrypt_block = (nettle_cipher_func*)gost28147_encrypt_for_cfb,
+	   .decrypt_block = (nettle_cipher_func*)gost28147_encrypt_for_cfb,
+
+	   .ctx_size = sizeof(struct CFB_CTX(struct gost28147_ctx, GOST28147_BLOCK_SIZE)),
+	   .encrypt = _cfb_encrypt,
+	   .decrypt = _cfb_decrypt,
+	   .set_encrypt_key = _gost28147_set_key_cpc,
+	   .set_decrypt_key = _gost28147_set_key_cpc,
+	},
+	{
+	   .algo = GNUTLS_CIPHER_GOST28147_CPD_CFB,
+	   .block_size = GOST28147_BLOCK_SIZE,
+	   .key_size = GOST28147_KEY_SIZE,
+	   .encrypt_block = (nettle_cipher_func*)gost28147_encrypt_for_cfb,
+	   .decrypt_block = (nettle_cipher_func*)gost28147_encrypt_for_cfb,
+
+	   .ctx_size = sizeof(struct CFB_CTX(struct gost28147_ctx, GOST28147_BLOCK_SIZE)),
+	   .encrypt = _cfb_encrypt,
+	   .decrypt = _cfb_decrypt,
+	   .set_encrypt_key = _gost28147_set_key_cpd,
+	   .set_decrypt_key = _gost28147_set_key_cpd,
+	},
+#endif
 };
 
 static int wrap_nettle_cipher_exists(gnutls_cipher_algorithm_t algo)
@@ -497,13 +623,7 @@ static int wrap_nettle_cipher_exists(gnutls_cipher_algorithm_t algo)
 	unsigned i;
 	for (i=0;i<sizeof(builtin_ciphers)/sizeof(builtin_ciphers[0]);i++) {
 		if (algo == builtin_ciphers[i].algo) {
-			if (_gnutls_fips_mode_enabled() == 0)
-				return 1;
-			else {
-				if (builtin_ciphers[i].fips_allowed)
-					return 1;
-				break;
-			}
+			return 1;
 		}
 	}
 	return 0;
@@ -529,9 +649,6 @@ wrap_nettle_cipher_init(gnutls_cipher_algorithm_t algo, void **_ctx,
 	if (idx == -1)
 		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
 
-	if (_gnutls_fips_mode_enabled() != 0 && builtin_ciphers[idx].fips_allowed == 0)
-		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
-
 	ctx = gnutls_calloc(1, sizeof(*ctx)+builtin_ciphers[idx].ctx_size+16);
 	if (ctx == NULL) {
 		gnutls_assert();
@@ -582,8 +699,7 @@ wrap_nettle_cipher_setiv(void *_ctx, const void *iv, size_t iv_size)
 	switch (ctx->cipher->algo) {
 	case GNUTLS_CIPHER_AES_128_GCM:
 	case GNUTLS_CIPHER_AES_256_GCM:
-		if (_gnutls_fips_mode_enabled() != 0 && iv_size < GCM_IV_SIZE)
-			return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+		FIPS_RULE(iv_size < GCM_IV_SIZE, GNUTLS_E_INVALID_REQUEST, "access to short GCM nonce size\n");
 		break;
 	case GNUTLS_CIPHER_SALSA20_256:
 	case GNUTLS_CIPHER_ESTREAM_SALSA20_256:
