@@ -24,17 +24,11 @@
 #ifndef DSA_FIPS_H_INCLUDED
 #define DSA_FIPS_H_INCLUDED
 
-#include <gmp.h>
+#include <nettle/bignum.h> /* includes gmp.h */
 #include <nettle/dsa.h>
 #include <nettle/sha2.h>
 #include <fips.h>
 
-#define MAX_PVP_SEED_SIZE 256
-
-#ifdef USE_NETTLE3
-# define dsa_public_key dsa_params
-#endif
-
 #define div_ceil(x,y) ((x+(y)-1)/(y))
 
 struct dss_params_validation_seeds {
@@ -58,7 +52,7 @@ st_provable_prime (mpz_t p,
 			  void *progress_ctx, nettle_progress_func * progress);
 
 int
-dsa_generate_dss_pqg(struct dsa_public_key *pub,
+dsa_generate_dss_pqg(struct dsa_params *params,
 		     struct dss_params_validation_seeds* cert,
 		     unsigned index,
 		     void *random_ctx, nettle_random_func *random,
@@ -66,27 +60,31 @@ dsa_generate_dss_pqg(struct dsa_public_key *pub,
 		     unsigned p_bits /* = L */, unsigned q_bits /* = N */);
 
 int
-dsa_generate_dss_keypair(struct dsa_public_key *pub,
-#ifdef USE_NETTLE3
+_dsa_generate_dss_pqg(struct dsa_params *params,
+			 struct dss_params_validation_seeds *cert,
+			 unsigned index,
+			 unsigned seed_size, void *seed,
+			 void *progress_ctx, nettle_progress_func * progress,
+			 unsigned p_bits /* = L */ , unsigned q_bits /* = N */ );
+
+int
+dsa_generate_dss_keypair(struct dsa_params *params,
 		     mpz_t y,
 		     mpz_t x,
-#else
-		     struct dsa_private_key *key,
-#endif
 		     void *random_ctx, nettle_random_func *random,
 		     void *progress_ctx, nettle_progress_func *progress);
 
 int
-dsa_validate_dss_pqg(struct dsa_public_key *pub,
+dsa_validate_dss_pqg(struct dsa_params *pub,
 		     struct dss_params_validation_seeds* cert,
 		     unsigned index);
 
 int
-_dsa_validate_dss_pq(struct dsa_public_key *pub,
+_dsa_validate_dss_pq(struct dsa_params *pub,
 		     struct dss_params_validation_seeds* cert);
 
 int
-_dsa_validate_dss_g(struct dsa_public_key *pub,
+_dsa_validate_dss_g(struct dsa_params *pub,
 		    unsigned domain_seed_size, const uint8_t *domain_seed, unsigned index);
 
 unsigned _dsa_check_qp_sizes(unsigned q_bits, unsigned p_bits, unsigned generate);
@@ -94,26 +92,22 @@ unsigned _dsa_check_qp_sizes(unsigned q_bits, unsigned p_bits, unsigned generate
 /* The following low-level functions can be used for DH key exchange as well 
  */
 int
-_dsa_generate_dss_pq(struct dsa_public_key *pub,
+_dsa_generate_dss_pq(struct dsa_params *pub,
 		     struct dss_params_validation_seeds* cert,
 		     unsigned seed_length, void* seed,
 		     void *progress_ctx, nettle_progress_func *progress,
 		     unsigned p_bits, unsigned q_bits);
 
 int
-_dsa_generate_dss_g(struct dsa_public_key *pub,
+_dsa_generate_dss_g(struct dsa_params *pub,
 		    unsigned domain_seed_size, const uint8_t* domain_seed,
 		    void *progress_ctx, nettle_progress_func * progress,
 		    unsigned index);
 
 void
-_dsa_generate_dss_xy(struct dsa_public_key *pub,
-#ifdef USE_NETTLE3
+_dsa_generate_dss_xy(struct dsa_params *pub,
 		     mpz_t y,
 		     mpz_t x,
-#else
-		     struct dsa_private_key *key,
-#endif
 		     void *random_ctx, nettle_random_func *random);
 
 #define DIGEST_SIZE SHA384_DIGEST_SIZE
