@@ -267,22 +267,9 @@ _rsa_generate_fips186_4_keypair(struct rsa_public_key *pub,
 	struct dss_params_validation_seeds cert;
 	unsigned l = n_size / 2;
 
-	if (_gnutls_fips_mode_enabled() != 0) {
-		if (n_size == 2048) {
-			if (seed_length != 14 * 2) {
-				_gnutls_debug_log("Seed length must be 28 bytes (it is %d)\n", seed_length);
-				return 0;
-			}
-		} else if (n_size == 3072) {
-			if (seed_length != 16 * 2) {
-				_gnutls_debug_log("Seed length must be 32 bytes (it is %d)\n", seed_length);
-				return 0;
-			}
-		} else {
-			_gnutls_debug_log("Unsupported size for modulus\n");
-			return 0;
-		}
-	}
+	FIPS_RULE(n_size == 2048 && seed_length != 14 * 2, 0, "seed length other than 28 bytes\n");
+	FIPS_RULE(n_size == 3072 && seed_length != 16 * 2, 0, "seed length other than 32 bytes\n");
+	FIPS_RULE(n_size != 2048 && n_size != 3072, 0, "unsupported size for modulus\n");
 
 	if (!mpz_tstbit(pub->e, 0)) {
 		_gnutls_debug_log("Unacceptable e (it is even)\n");
@@ -420,12 +407,7 @@ rsa_generate_fips186_4_keypair(struct rsa_public_key *pub,
 	unsigned seed_length;
 	int ret;
 
-	if (_gnutls_fips_mode_enabled() != 0) {
-		if (n_size != 2048 && n_size != 3072) {
-			_gnutls_debug_log("The size of a prime can only be 2048 or 3072\n");
-			return 0;
-		}
-	}
+	FIPS_RULE(n_size != 2048 && n_size != 3072, 0, "size of prime of other than 2048 or 3072\n");
 
 	seed_length = SEED_LENGTH(n_size);
 	if (seed_length > sizeof(seed))
