@@ -46,7 +46,6 @@
 #include "record.h"
 #include "datum.h"
 #include "constate.h"
-#include "ext/max_record.h"
 #include "tls13/key_update.h"
 #include <ext/heartbeat.h>
 #include <state.h>
@@ -293,7 +292,7 @@ int gnutls_bye(gnutls_session_t session, gnutls_close_request_t how)
 			gnutls_assert();
 			return ret;
 		}
-		/* fall through */
+		FALLTHROUGH;
 	case BYE_STATE1:
 		ret =
 		    gnutls_alert_send(session, GNUTLS_AL_WARNING,
@@ -303,7 +302,7 @@ int gnutls_bye(gnutls_session_t session, gnutls_close_request_t how)
 			gnutls_assert();
 			return ret;
 		}
-		/* fall through */
+		FALLTHROUGH;
 	case BYE_STATE2:
 		BYE_STATE = BYE_STATE2;
 		if (how == GNUTLS_SHUT_RDWR) {
@@ -1190,8 +1189,15 @@ static int recv_headers(gnutls_session_t session,
 		    (session, "Received packet with illegal length: %u\n",
 		     (unsigned int) record->length);
 
-		if (record->length == 0)
+		if (record->length == 0) {
+			/* Empty, unencrypted records are always unexpected. */
+			if (record_params->cipher->id == GNUTLS_CIPHER_NULL)
+				return
+				    gnutls_assert_val
+				    (GNUTLS_E_UNEXPECTED_PACKET);
+
 			return gnutls_assert_val(GNUTLS_E_DECRYPTION_FAILED);
+		}
 		return
 		    gnutls_assert_val(GNUTLS_E_RECORD_OVERFLOW);
 	}
@@ -1334,6 +1340,44 @@ _gnutls_recv_in_buffers(gnutls_session_t session, content_type_t type,
 
 	_mbuffer_head_remove_bytes(&session->internals.record_recv_buffer,
 				   record.header_size + record.length);
+
+	/* FIXME: as 0-RTT is not implemented yet, when early data is
+	 * indicated, skip decryption failure up to
+	 * max_early_data_size. Otherwise, if the record is properly
+	 * decrypted, treat it as the start of client's second flight.
+	 *
+	 * This implements the first way suggested in 4.2.10 of
+	 * draft-ietf-tls-tls13-28.
+	 */
+	if (unlikely(session->internals.hsk_flags & HSK_EARLY_DATA_IN_FLIGHT)) {
+		if (record.type == GNUTLS_APPLICATION_DATA &&
+		    (ret < 0 ||
+		     /* early data must always be encrypted, treat it
+		      * as decryption failure if otherwise */
+		     record_params->cipher->id == GNUTLS_CIPHER_NULL)) {
+			if (record.length >
+			    session->security_parameters.max_early_data_size -
+			    session->internals.early_data_received) {
+				_gnutls_record_log
+					("REC[%p]: max_early_data_size exceeded\n",
+					 session);
+				ret = GNUTLS_E_UNEXPECTED_PACKET;
+				goto sanity_check_error;
+			}
+
+			_gnutls_record_log("REC[%p]: Discarded early data[%u] due to invalid decryption, length: %u\n",
+					   session,
+					   (unsigned int)
+					   _gnutls_uint64touint32(packet_sequence),
+					   (unsigned int)
+					   record.length);
+			session->internals.early_data_received += record.length;
+			goto discard;
+		} else {
+			session->internals.hsk_flags &= ~HSK_EARLY_DATA_IN_FLIGHT;
+		}
+	}
+
 	if (ret < 0) {
 		gnutls_assert();
 		_gnutls_audit_log(session,
@@ -1521,30 +1565,48 @@ check_session_status(gnutls_session_t session, unsigned ms)
 			return gnutls_assert_val(ret);
 
 		return GNUTLS_E_AGAIN;
+	case RECV_STATE_EARLY_START_HANDLING:
 	case RECV_STATE_FALSE_START_HANDLING:
 		return 1;
 	case RECV_STATE_FALSE_START:
 		/* if false start is not complete we always expect for handshake packets
 		 * prior to anything else. */
-		if (session->security_parameters.entity == GNUTLS_CLIENT &&
-		    (session->internals.flags & GNUTLS_ENABLE_FALSE_START)) {
-			/* Attempt to complete handshake */
-
-			session->internals.recv_state = RECV_STATE_FALSE_START_HANDLING;
-			ret = gnutls_handshake(session);
-			if (ret < 0) {
-				/* a temp or fatal error, make sure we reset the state
-				 * so we can resume or temp errors */
-				session->internals.recv_state = RECV_STATE_FALSE_START;
-				gnutls_assert();
-				return ret;
-			}
-
-			session->internals.recv_state = RECV_STATE_0;
-			return 1;
-		} else {
+		if (session->security_parameters.entity != GNUTLS_CLIENT ||
+		    !(session->internals.flags & GNUTLS_ENABLE_FALSE_START))
 			return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
+
+		/* Attempt to complete handshake */
+
+		session->internals.recv_state = RECV_STATE_FALSE_START_HANDLING;
+		ret = gnutls_handshake(session);
+		if (ret < 0) {
+			/* a temp or fatal error, make sure we reset the state
+			 * so we can resume on temp errors */
+			session->internals.recv_state = RECV_STATE_FALSE_START;
+			return gnutls_assert_val(ret);
 		}
+
+		session->internals.recv_state = RECV_STATE_0;
+		return 1;
+	case RECV_STATE_EARLY_START:
+		/* if early start is not complete we always expect for handshake packets
+		 * prior to anything else. */
+		if (session->security_parameters.entity != GNUTLS_SERVER ||
+		    !(session->internals.flags & GNUTLS_ENABLE_EARLY_START))
+			return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
+
+		/* Attempt to complete handshake */
+		session->internals.recv_state = RECV_STATE_EARLY_START_HANDLING;
+		ret = gnutls_handshake(session);
+		if (ret < 0) {
+			/* a temp or fatal error, make sure we reset the state
+			 * so we can resume on temp errors */
+			session->internals.recv_state = RECV_STATE_EARLY_START;
+			return gnutls_assert_val(ret);
+		}
+
+		session->internals.recv_state = RECV_STATE_0;
+		return 1;
 	case RECV_STATE_DTLS_RETRANSMIT:
 		ret = _dtls_retransmit(session);
 		if (ret < 0)
@@ -1552,7 +1614,6 @@ check_session_status(gnutls_session_t session, unsigned ms)
 
 		session->internals.recv_state = RECV_STATE_0;
 
-		/* Fall through: */
 		FALLTHROUGH;
 	case RECV_STATE_0:
 
@@ -1809,7 +1870,8 @@ gnutls_record_send2(gnutls_session_t session, const void *data,
 		/* this is to protect buggy applications from sending unencrypted
 		 * data. We allow sending however, if we are in false start handshake
 		 * state. */
-		if (session->internals.recv_state != RECV_STATE_FALSE_START)
+		if (session->internals.recv_state != RECV_STATE_FALSE_START &&
+		    session->internals.recv_state != RECV_STATE_EARLY_START)
 			return gnutls_assert_val(GNUTLS_E_UNAVAILABLE_DURING_HANDSHAKE);
 	}
 
@@ -1839,14 +1901,14 @@ gnutls_record_send2(gnutls_session_t session, const void *data,
 				return gnutls_assert_val(ret);
 
 			session->internals.rsend_state = RECORD_SEND_KEY_UPDATE_2;
-			/* fall-through */
+			FALLTHROUGH;
 		case RECORD_SEND_KEY_UPDATE_2:
 			ret = gnutls_session_key_update(session, 0);
 			if (ret < 0)
 				return gnutls_assert_val(ret);
 
 			session->internals.rsend_state = RECORD_SEND_KEY_UPDATE_3;
-			/* fall-through */
+			FALLTHROUGH;
 		case RECORD_SEND_KEY_UPDATE_3:
 			ret = _gnutls_send_int(session, GNUTLS_APPLICATION_DATA,
 						-1, EPOCH_WRITE_CURRENT,
@@ -1984,7 +2046,8 @@ gnutls_record_recv(gnutls_session_t session, void *data, size_t data_size)
 		/* this is to protect buggy applications from sending unencrypted
 		 * data. We allow sending however, if we are in false start handshake
 		 * state. */
-		if (session->internals.recv_state != RECV_STATE_FALSE_START)
+		if (session->internals.recv_state != RECV_STATE_FALSE_START &&
+		    session->internals.recv_state != RECV_STATE_EARLY_START)
 			return gnutls_assert_val(GNUTLS_E_UNAVAILABLE_DURING_HANDSHAKE);
 	}
 
