@@ -860,7 +860,8 @@ pack_security_parameters(gnutls_session_t session, gnutls_buffer_st * ps)
 	size_t cur_size;
 
 	if (session->security_parameters.epoch_read
-	    != session->security_parameters.epoch_write) {
+	    != session->security_parameters.epoch_write &&
+	    !(session->internals.hsk_flags & HSK_EARLY_START_USED)) {
 		gnutls_assert();
 		return GNUTLS_E_UNAVAILABLE_DURING_HANDSHAKE;
 	}
@@ -885,6 +886,11 @@ pack_security_parameters(gnutls_session_t session, gnutls_buffer_st * ps)
 	BUFFER_APPEND_NUM(ps,
 			  session->security_parameters.server_auth_type);
 
+	BUFFER_APPEND(ps, &session->security_parameters.session_id_size,
+		      1);
+	BUFFER_APPEND(ps, session->security_parameters.session_id,
+		      session->security_parameters.session_id_size);
+
 	BUFFER_APPEND_NUM(ps, session->security_parameters.pversion->id);
 
 	/* if we are under TLS 1.3 do not pack keys or params negotiated using an extension
@@ -892,7 +898,8 @@ pack_security_parameters(gnutls_session_t session, gnutls_buffer_st * ps)
 	if (!session->security_parameters.pversion->tls13_sem) {
 		BUFFER_APPEND(ps, session->security_parameters.cs->id, 2);
 
-		BUFFER_APPEND_NUM(ps, session->security_parameters.cert_type);
+		BUFFER_APPEND_NUM(ps, session->security_parameters.client_ctype);
+		BUFFER_APPEND_NUM(ps, session->security_parameters.server_ctype);
 
 		BUFFER_APPEND_PFX1(ps, session->security_parameters.master_secret,
 			      GNUTLS_MASTER_SIZE);
@@ -901,11 +908,6 @@ pack_security_parameters(gnutls_session_t session, gnutls_buffer_st * ps)
 		BUFFER_APPEND_PFX1(ps, session->security_parameters.server_random,
 			      GNUTLS_RANDOM_SIZE);
 
-		BUFFER_APPEND(ps, &session->security_parameters.session_id_size,
-			      1);
-		BUFFER_APPEND(ps, session->security_parameters.session_id,
-			      session->security_parameters.session_id_size);
-
 		BUFFER_APPEND_NUM(ps,
 				  session->security_parameters.
 				  max_record_send_size);
@@ -958,8 +960,6 @@ unpack_security_parameters(gnutls_session_t session, gnutls_buffer_st * ps)
 	session->internals.resumed_security_parameters.timestamp =
 	    timestamp;
 
-	timestamp = gnutls_time(0);
-
 	BUFFER_POP_NUM(ps,
 		       session->internals.resumed_security_parameters.
 		       entity);
@@ -976,6 +976,16 @@ unpack_security_parameters(gnutls_session_t session, gnutls_buffer_st * ps)
 		       session->internals.resumed_security_parameters.
 		       server_auth_type);
 
+	BUFFER_POP(ps,
+		   &session->internals.resumed_security_parameters.
+		   session_id_size, 1);
+
+	BUFFER_POP(ps,
+		   session->internals.resumed_security_parameters.
+		   session_id,
+		   session->internals.resumed_security_parameters.
+		   session_id_size);
+
 	BUFFER_POP_NUM(ps, version);
 	session->internals.resumed_security_parameters.pversion =
 	    version_to_entry(version);
@@ -990,8 +1000,11 @@ unpack_security_parameters(gnutls_session_t session, gnutls_buffer_st * ps)
 			return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
 
 		BUFFER_POP_NUM(ps,
-			       session->internals.resumed_security_parameters.
-			       cert_type);
+				session->internals.resumed_security_parameters.
+				client_ctype);
+		BUFFER_POP_NUM(ps,
+				session->internals.resumed_security_parameters.
+				server_ctype);
 
 		/* master secret */
 		ret = _gnutls_buffer_pop_datum_prefix8(ps, &t);
@@ -1023,15 +1036,6 @@ unpack_security_parameters(gnutls_session_t session, gnutls_buffer_st * ps)
 		if (t.size == GNUTLS_RANDOM_SIZE)
 			memcpy(session->internals.resumed_security_parameters.server_random, t.data, t.size);
 
-		BUFFER_POP(ps,
-			   &session->internals.resumed_security_parameters.
-			   session_id_size, 1);
-
-		BUFFER_POP(ps,
-			   session->internals.resumed_security_parameters.
-			   session_id,
-			   session->internals.resumed_security_parameters.
-			   session_id_size);
 
 		BUFFER_POP_NUM(ps,
 			       session->internals.resumed_security_parameters.
@@ -1065,15 +1069,6 @@ unpack_security_parameters(gnutls_session_t session, gnutls_buffer_st * ps)
 		}
 	}
 
-	if (timestamp -
-	    session->internals.resumed_security_parameters.timestamp >
-	    session->internals.expire_time
-	    || session->internals.resumed_security_parameters.timestamp >
-	    timestamp) {
-		gnutls_assert();
-		return GNUTLS_E_EXPIRED;
-	}
-
 	ret = 0;
 
       error:
@@ -1124,13 +1119,18 @@ gnutls_session_set_premaster(gnutls_session_t session, unsigned int entity,
 		return gnutls_assert_val(ret);
 
 	session->internals.resumed_security_parameters.cs = ciphersuite_to_entry(cs);
-	if (cs == NULL)
+	if (session->internals.resumed_security_parameters.cs == NULL)
 		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
 
-	session->internals.resumed_security_parameters.cert_type =
-	    DEFAULT_CERT_TYPE;
+	session->internals.resumed_security_parameters.client_ctype =
+			DEFAULT_CERT_TYPE;
+	session->internals.resumed_security_parameters.server_ctype =
+			DEFAULT_CERT_TYPE;
 	session->internals.resumed_security_parameters.pversion =
 	    version_to_entry(version);
+	if (session->internals.resumed_security_parameters.pversion ==
+	    NULL)
+		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
 
 	if (session->internals.resumed_security_parameters.pversion->selectable_prf)
 		session->internals.resumed_security_parameters.prf = mac_to_entry(session->internals.resumed_security_parameters.cs->prf);
@@ -1139,10 +1139,6 @@ gnutls_session_set_premaster(gnutls_session_t session, unsigned int entity,
 	if (session->internals.resumed_security_parameters.prf == NULL)
 		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
 
-	if (session->internals.resumed_security_parameters.pversion ==
-	    NULL)
-		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
-
 	if (master->size != GNUTLS_MASTER_SIZE)
 		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
 
