@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2000-2012 Free Software Foundation, Inc.
+ * Copyright (C) 2016-2017 Red Hat, Inc.
  *
  * Author: Nikos Mavrogiannopoulos
  *
@@ -100,28 +101,53 @@ int _gnutls_buffer_append_str(gnutls_buffer_st *, const char *str);
 
 #include <num.h>
 
-void _gnutls_buffer_replace_data(gnutls_buffer_st * buf,
-				 gnutls_datum_t * data);
-
 int _gnutls_buffer_append_prefix(gnutls_buffer_st * buf, int pfx_size,
 				 size_t data_size);
 
 int _gnutls_buffer_append_mpi(gnutls_buffer_st * buf, int pfx_size,
 			      bigint_t, int lz);
 
+int _gnutls_buffer_append_fixed_mpi(gnutls_buffer_st * buf,
+				    bigint_t mpi, unsigned size);
+
 int _gnutls_buffer_append_data_prefix(gnutls_buffer_st * buf, int pfx_size,
 				      const void *data, size_t data_size);
-void _gnutls_buffer_pop_data(gnutls_buffer_st *, void *, size_t * size);
+int _gnutls_buffer_pop_data(gnutls_buffer_st *, void *, size_t size);
 void _gnutls_buffer_pop_datum(gnutls_buffer_st *, gnutls_datum_t *,
 			      size_t max_size);
 
-int _gnutls_buffer_pop_prefix(gnutls_buffer_st * buf, size_t * data_size,
-			      int check);
+int _gnutls_buffer_pop_prefix8(gnutls_buffer_st *, uint8_t *, int check);
+
+/* 32-bit prefix */
+int _gnutls_buffer_pop_prefix32(gnutls_buffer_st * buf, size_t * data_size,
+			        int check);
+
+int _gnutls_buffer_pop_prefix24(gnutls_buffer_st * buf, size_t * data_size,
+			        int check);
+
+/* 32-bit prefix */
+int _gnutls_buffer_pop_datum_prefix32(gnutls_buffer_st * buf,
+				      gnutls_datum_t * data);
+
+/* 16-bit prefix */
+int _gnutls_buffer_pop_datum_prefix16(gnutls_buffer_st * buf,
+				      gnutls_datum_t * data);
+
+/* 8-bit prefix */
+int _gnutls_buffer_pop_datum_prefix8(gnutls_buffer_st * buf,
+				     gnutls_datum_t * data);
 
-int _gnutls_buffer_pop_datum_prefix(gnutls_buffer_st * buf,
-				    gnutls_datum_t * data);
 int _gnutls_buffer_to_datum(gnutls_buffer_st * str, gnutls_datum_t * data, unsigned is_str);
 
+inline static
+void _gnutls_ro_buffer_from_datum(gnutls_buffer_st * str, gnutls_datum_t * data)
+{
+	_gnutls_buffer_init(str);
+	str->length = data->size;
+	str->max_length = data->size;
+	str->data = data->data;
+}
+
 int
 _gnutls_buffer_append_escape(gnutls_buffer_st * dest, const void *data,
 			     size_t data_size, const char *invalid_chars);
@@ -208,9 +234,8 @@ int _gnutls_hostname_compare(const char *certname, size_t certnamesize,
     }
 
 #define BUFFER_POP(b, x, s) { \
-	size_t is = s; \
-	_gnutls_buffer_pop_data(b, x, &is); \
-	if (is != s) { \
+	ret = _gnutls_buffer_pop_data(b, x, s); \
+	if (ret < 0) { \
 	    ret = GNUTLS_E_PARSING_ERROR; \
 	    gnutls_assert(); \
 	    goto error; \
@@ -219,7 +244,7 @@ int _gnutls_hostname_compare(const char *certname, size_t certnamesize,
 
 #define BUFFER_POP_DATUM(b, o) { \
 	gnutls_datum_t d; \
-	ret = _gnutls_buffer_pop_datum_prefix(b, &d); \
+	ret = _gnutls_buffer_pop_datum_prefix32(b, &d); \
 	if (ret >= 0) \
 	    ret = _gnutls_set_datum (o, d.data, d.size); \
 	if (ret < 0) { \
@@ -230,7 +255,7 @@ int _gnutls_hostname_compare(const char *certname, size_t certnamesize,
 
 #define BUFFER_POP_NUM(b, o) { \
 	size_t s; \
-	ret = _gnutls_buffer_pop_prefix(b, &s, 0); \
+	ret = _gnutls_buffer_pop_prefix32(b, &s, 0); \
 	if (ret < 0) { \
 	    gnutls_assert(); \
 	    goto error; \
@@ -240,7 +265,7 @@ int _gnutls_hostname_compare(const char *certname, size_t certnamesize,
 
 #define BUFFER_POP_CAST_NUM(b, o) { \
 	size_t s; \
-	ret = _gnutls_buffer_pop_prefix(b, &s, 0); \
+	ret = _gnutls_buffer_pop_prefix32(b, &s, 0); \
 	if (ret < 0) { \
 	    gnutls_assert(); \
 	    goto error; \
