@@ -39,8 +39,8 @@ static void tls_log_func(int level, const char *str)
 
 static unsigned page_size = 4096;
 
-#define ALLOC(x) x=malloc(step+64)
-#define ALLOCM(x, mem) x=malloc(mem); assert(gnutls_rnd(GNUTLS_RND_NONCE, x, mem) >= 0)
+#define ALLOC(x) {x=malloc(step+64);assert(x!=NULL);}
+#define ALLOCM(x, mem) {x=malloc(mem); assert(x!=NULL); assert(gnutls_rnd(GNUTLS_RND_NONCE, x, mem) >= 0);}
 #define FREE(x) free(x)
 #define INC(orig, x, s) x+=page_size; if ((x+step) >= (((unsigned char*)orig) + MAX_MEM)) { x = orig; }
 
@@ -67,8 +67,7 @@ static void cipher_mac_bench(int algo, int mac_algo, int size)
 
 	_iv = malloc(ivsize);
 	if (_iv == NULL) {
-		if (_key)
-			free(_key);
+		free(_key);
 		return;
 	}
 	memset(_iv, 0xf0, ivsize);
@@ -115,10 +114,10 @@ static void cipher_mac_bench(int algo, int mac_algo, int size)
 	gnutls_hmac_deinit(mac_ctx, NULL);
 
 	stop_benchmark(&st, NULL, 1);
-	FREE(input);
-	FREE(output);
 
       leave:
+	FREE(input);
+	FREE(output);
 	free(_key);
 	free(_iv);
 }
@@ -155,8 +154,10 @@ static void cipher_bench(int algo, int size, int aead)
 	memset(_key, 0xf0, keysize);
 
 	_iv = malloc(ivsize);
-	if (_iv == NULL)
+	if (_iv == NULL) {
+		free(_key);
 		return;
+	}
 	memset(_iv, 0xf0, ivsize);
 
 	iv.data = _iv;
@@ -274,12 +275,12 @@ void benchmark_cipher(int debug_level)
 	page_size = sysconf(_SC_PAGESIZE);
 #endif
 
-	printf("Checking cipher-MAC combinations, payload size: %u\n", size * 1024);
+	printf("Checking AEAD ciphers, payload size: %u\n", size * 1024);
 	cipher_bench(GNUTLS_CIPHER_AES_128_GCM, size, 1);
 	cipher_bench(GNUTLS_CIPHER_AES_128_CCM, size, 1);
 	cipher_bench(GNUTLS_CIPHER_CHACHA20_POLY1305, size, 1);
-	cipher_bench(GNUTLS_CIPHER_NULL, size, 1);
 
+	printf("\nChecking cipher-MAC combinations, payload size: %u\n", size * 1024);
 	cipher_mac_bench(GNUTLS_CIPHER_SALSA20_256, GNUTLS_MAC_SHA1, size);
 	cipher_mac_bench(GNUTLS_CIPHER_AES_128_CBC, GNUTLS_MAC_SHA1, size);
 	cipher_mac_bench(GNUTLS_CIPHER_AES_128_CBC, GNUTLS_MAC_SHA256,
@@ -293,8 +294,8 @@ void benchmark_cipher(int debug_level)
 	printf("\nChecking ciphers, payload size: %u\n", size * 1024);
 	cipher_bench(GNUTLS_CIPHER_3DES_CBC, size, 0);
 	cipher_bench(GNUTLS_CIPHER_AES_128_CBC, size, 0);
-	cipher_bench(GNUTLS_CIPHER_ARCFOUR, size, 0);
 	cipher_bench(GNUTLS_CIPHER_SALSA20_256, size, 0);
+	cipher_bench(GNUTLS_CIPHER_NULL, size, 1);
 
 	gnutls_global_deinit();
 }
