@@ -809,14 +809,20 @@ static void print_head(FILE * out, const char *txt, unsigned int size,
 {
 	unsigned i;
 	char *p, *ntxt;
+	int ret;
 
 	if (cprint != 0) {
 		if (size > 0)
-			asprintf(&ntxt, "const unsigned char %s[%u] =",
-				 txt, size);
+			ret = asprintf(&ntxt, "const unsigned char %s[%u] =",
+				       txt, size);
 		else
-			asprintf(&ntxt, "const unsigned char %s[] =\n",
-				 txt);
+			ret = asprintf(&ntxt, "const unsigned char %s[] =\n",
+				       txt);
+
+		if (ret == -1) {
+			fprintf(stderr, "memory error\n");
+			app_exit(1);
+		}
 
 		p = strstr(ntxt, "char");
 		p += 5;
@@ -907,22 +913,15 @@ print_ecc_pkey(FILE * outfile, gnutls_ecc_curve_t curve,
 	}
 }
 
-static const char *
-gost_param_name(int param)
+static void reverse_datum(gnutls_datum_t *d)
 {
-	switch(param) {
-	case 0:
-		return "TC26-Z";
-	case 1:
-		return "CryptoPro-A";
-	case 2:
-		return "CryptoPro-B";
-	case 3:
-		return "CryptoPro-C";
-	case 4:
-		return "CryptoPro-D";
-	default:
-		return "unknown";
+	unsigned int i;
+	unsigned char c;
+
+	for (i = 0; i < d->size / 2; i++) {
+		c = d->data[i];
+		d->data[i] = d->data[d->size - i - 1];
+		d->data[d->size - i - 1] = c;
 	}
 }
 
@@ -948,15 +947,18 @@ print_gost_pkey(FILE * outfile, gnutls_ecc_curve_t curve,
 
 	if (cprint != 0)
 		fprintf(outfile, "/* paramset: %s */\n",
-			gost_param_name(paramset));
+			gnutls_gost_paramset_get_name(paramset));
 	else
 		fprintf(outfile, "paramset:\t%s\n",
-			gost_param_name(paramset));
+			gnutls_gost_paramset_get_name(paramset));
 
 	if (k) {
+		reverse_datum(k);
 		print_head(outfile, "private key", k->size, cprint);
 		print_hex_datum(outfile, k, cprint);
 	}
+	reverse_datum(x);
+	reverse_datum(y);
 	print_head(outfile, "x", x->size, cprint);
 	print_hex_datum(outfile, x, cprint);
 	print_head(outfile, "y", y->size, cprint);
@@ -1517,6 +1519,10 @@ int generate_prime(FILE * outfile, int how, common_info_st * info)
 			p = gnutls_ffdhe_4096_group_prime;
 			g = gnutls_ffdhe_4096_group_generator;
 			key_bits = gnutls_ffdhe_4096_key_bits;
+		} else if (bits <= 6144) {
+			p = gnutls_ffdhe_6144_group_prime;
+			g = gnutls_ffdhe_6144_group_generator;
+			key_bits = gnutls_ffdhe_6144_key_bits;
 		} else {
 			p = gnutls_ffdhe_8192_group_prime;
 			g = gnutls_ffdhe_8192_group_generator;
