@@ -796,7 +796,6 @@ generate_crl(gnutls_x509_crt_t ca_crt, common_info_st * cinfo)
 		app_exit(1);
 	}
 
-	fprintf(stderr, "Update times.\n");
 	secs = get_crl_next_update();
 
 	result =
@@ -884,7 +883,11 @@ static gnutls_digest_algorithm_t get_dig(gnutls_x509_crt_t crt, common_info_st *
 	gnutls_pubkey_t pubkey;
 	int result;
 
-	gnutls_pubkey_init(&pubkey);
+	result = gnutls_pubkey_init(&pubkey);
+	if (result < 0) {
+		fprintf(stderr, "memory error\n");
+		app_exit(1);
+	}
 
 	result = gnutls_pubkey_import_x509(pubkey, crt, 0);
 	if (result < 0) {
@@ -905,8 +908,8 @@ static gnutls_digest_algorithm_t get_dig(gnutls_x509_crt_t crt, common_info_st *
 void generate_self_signed(common_info_st * cinfo)
 {
 	gnutls_x509_crt_t crt;
+	gnutls_datum_t out;
 	gnutls_privkey_t key;
-	size_t size;
 	int result;
 	unsigned int flags = 0;
 
@@ -933,15 +936,15 @@ void generate_self_signed(common_info_st * cinfo)
 		app_exit(1);
 	}
 
-	size = lbuffer_size;
 	result =
-	    gnutls_x509_crt_export(crt, outcert_format, lbuffer, &size);
+	    gnutls_x509_crt_export2(crt, outcert_format, &out);
 	if (result < 0) {
 		fprintf(stderr, "crt_export: %s\n", gnutls_strerror(result));
 		app_exit(1);
 	}
 
-	fwrite(lbuffer, 1, size, outfile);
+	fwrite(out.data, 1, out.size, outfile);
+	gnutls_free(out.data);
 
 	gnutls_x509_crt_deinit(crt);
 	gnutls_privkey_deinit(key);
@@ -951,7 +954,7 @@ static void generate_signed_certificate(common_info_st * cinfo)
 {
 	gnutls_x509_crt_t crt;
 	gnutls_privkey_t key;
-	size_t size;
+	gnutls_datum_t out;
 	int result;
 	gnutls_privkey_t ca_key;
 	gnutls_x509_crt_t ca_crt;
@@ -985,15 +988,15 @@ static void generate_signed_certificate(common_info_st * cinfo)
 		app_exit(1);
 	}
 
-	size = lbuffer_size;
 	result =
-	    gnutls_x509_crt_export(crt, outcert_format, lbuffer, &size);
+	    gnutls_x509_crt_export2(crt, outcert_format, &out);
 	if (result < 0) {
 		fprintf(stderr, "crt_export: %s\n", gnutls_strerror(result));
 		app_exit(1);
 	}
 
-	fwrite(lbuffer, 1, size, outfile);
+	fwrite(out.data, 1, out.size, outfile);
+	gnutls_free(out.data);
 
 	gnutls_x509_crt_deinit(crt);
 	gnutls_x509_crt_deinit(ca_crt);
@@ -1005,7 +1008,7 @@ static void generate_proxy_certificate(common_info_st * cinfo)
 {
 	gnutls_x509_crt_t crt, eecrt;
 	gnutls_privkey_t key, eekey;
-	size_t size;
+	gnutls_datum_t out;
 	int result;
 	unsigned int flags = 0;
 
@@ -1031,15 +1034,15 @@ static void generate_proxy_certificate(common_info_st * cinfo)
 		app_exit(1);
 	}
 
-	size = lbuffer_size;
 	result =
-	    gnutls_x509_crt_export(crt, outcert_format, lbuffer, &size);
+	    gnutls_x509_crt_export2(crt, outcert_format, &out);
 	if (result < 0) {
 		fprintf(stderr, "crt_export: %s\n", gnutls_strerror(result));
 		app_exit(1);
 	}
 
-	fwrite(lbuffer, 1, size, outfile);
+	fwrite(out.data, 1, out.size, outfile);
+	gnutls_free(out.data);
 
 	gnutls_x509_crt_deinit(eecrt);
 	gnutls_x509_crt_deinit(crt);
@@ -1080,10 +1083,10 @@ static void generate_signed_crl(common_info_st * cinfo)
 static void update_signed_certificate(common_info_st * cinfo)
 {
 	gnutls_x509_crt_t crt;
-	size_t size;
 	int result;
 	gnutls_privkey_t ca_key;
 	gnutls_x509_crt_t ca_crt;
+	gnutls_datum_t out;
 	time_t tim;
 	unsigned int flags = 0;
 
@@ -1125,15 +1128,15 @@ static void update_signed_certificate(common_info_st * cinfo)
 		app_exit(1);
 	}
 
-	size = lbuffer_size;
 	result =
-	    gnutls_x509_crt_export(crt, outcert_format, lbuffer, &size);
+	    gnutls_x509_crt_export2(crt, outcert_format, &out);
 	if (result < 0) {
 		fprintf(stderr, "crt_export: %s\n", gnutls_strerror(result));
 		app_exit(1);
 	}
 
-	fwrite(lbuffer, 1, size, outfile);
+	fwrite(out.data, 1, out.size, outfile);
+	gnutls_free(out.data);
 
 	gnutls_x509_crt_deinit(crt);
 }
@@ -1451,6 +1454,7 @@ void certificate_info(int pubkey, common_info_st * cinfo)
 {
 	gnutls_x509_crt_t *crts = NULL;
 	size_t size;
+	gnutls_datum_t out;
 	int ret, i, count;
 	gnutls_datum_t pem;
 	unsigned int crt_num;
@@ -1492,17 +1496,16 @@ void certificate_info(int pubkey, common_info_st * cinfo)
 			/* this deinitializes the certificate */
 			pubkey_info(crts[i], cinfo);
 		} else {
-			size = lbuffer_size;
 			ret =
-			    gnutls_x509_crt_export(crts[i], outcert_format,
-						   lbuffer, &size);
+			    gnutls_x509_crt_export2(crts[i], outcert_format, &out);
 			if (ret < 0) {
 				fprintf(stderr, "export error: %s\n",
 					gnutls_strerror(ret));
 				app_exit(1);
 			}
 
-			fwrite(lbuffer, 1, size, outfile);
+			fwrite(out.data, 1, out.size, outfile);
+			gnutls_free(out.data);
 
 			gnutls_x509_crt_deinit(crts[i]);
 		}
@@ -1682,7 +1685,11 @@ void privkey_info(common_info_st * cinfo)
 	size = fread(lbuffer, 1, lbuffer_size - 1, infile);
 	lbuffer[size] = 0;
 
-	gnutls_x509_privkey_init(&key);
+	ret = gnutls_x509_privkey_init(&key);
+	if (ret < 0) {
+		fprintf(stderr, "privkey_init: %s", gnutls_strerror(ret));
+		app_exit(1);
+	}
 
 	pem.data = lbuffer;
 	pem.size = size;
@@ -1736,7 +1743,11 @@ static void privkey_to_rsa(common_info_st * cinfo)
 	size = fread(lbuffer, 1, lbuffer_size - 1, infile);
 	lbuffer[size] = 0;
 
-	gnutls_x509_privkey_init(&key);
+	ret = gnutls_x509_privkey_init(&key);
+	if (ret < 0) {
+		fprintf(stderr, "privkey_init: %s", gnutls_strerror(ret));
+		app_exit(1);
+	}
 
 	pem.data = lbuffer;
 	pem.size = size;
@@ -2467,7 +2478,7 @@ void verify_crl(common_info_st * cinfo)
 	size_t size;
 	gnutls_datum_t dn;
 	unsigned int output;
-	int ret;
+	int ret, rc;
 	gnutls_datum_t pem, pout;
 	gnutls_x509_crl_t crl;
 	gnutls_x509_crt_t issuer;
@@ -2507,7 +2518,6 @@ void verify_crl(common_info_st * cinfo)
 
 	print_crl_info(crl, outfile);
 
-	fprintf(outfile, "Verification output: ");
 	ret = gnutls_x509_crl_verify(crl, &issuer, 1, 0, &output);
 	if (ret < 0) {
 		fprintf(stderr, "verification error: %s\n",
@@ -2515,10 +2525,14 @@ void verify_crl(common_info_st * cinfo)
 		app_exit(1);
 	}
 
+	fprintf(outfile, "Verification output: ");
+
 	if (output) {
 		fprintf(outfile, "Not verified. ");
+		rc = 1;
 	} else {
 		fprintf(outfile, "Verified.");
+		rc = 0;
 	}
 
 	ret =
@@ -2534,6 +2548,8 @@ void verify_crl(common_info_st * cinfo)
 	gnutls_free(pout.data);
 
 	fprintf(outfile, "\n");
+
+	app_exit(rc);
 }
 
 static void print_dn(const char *prefix, const gnutls_datum_t *raw)
