@@ -110,7 +110,7 @@ pkcs11_delete(FILE * outfile, const char *url,
 }
 
 static
-const char *get_key_algo_type(gnutls_pkcs11_obj_type_t otype, const char *objurl, unsigned flags)
+const char *get_key_algo_type(gnutls_pkcs11_obj_type_t otype, const char *objurl, unsigned flags, time_t *exp)
 {
 	int ret;
 	gnutls_pubkey_t pubkey = NULL;
@@ -122,9 +122,15 @@ const char *get_key_algo_type(gnutls_pkcs11_obj_type_t otype, const char *objurl
 	gnutls_pk_algorithm_t pk;
 	gnutls_ecc_curve_t curve;
 
+	if (exp)
+		*exp = -1;
+
 	switch (otype) {
 		case GNUTLS_PKCS11_OBJ_X509_CRT:
-			gnutls_x509_crt_init(&crt);
+			ret = gnutls_x509_crt_init(&crt);
+			if (ret < 0)
+				goto fail;
+
 			ret = gnutls_x509_crt_import_url(crt, objurl, flags);
 			if (ret < 0)
 				goto fail;
@@ -143,10 +149,17 @@ const char *get_key_algo_type(gnutls_pkcs11_obj_type_t otype, const char *objurl
 					p = str;
 				}
 			}
+
+			if (exp)
+				*exp = gnutls_x509_crt_get_expiration_time(crt);
+
 			gnutls_x509_crt_deinit(crt);
 			return p;
 		case GNUTLS_PKCS11_OBJ_PUBKEY:
-			gnutls_pubkey_init(&pubkey);
+			ret = gnutls_pubkey_init(&pubkey);
+			if (ret < 0)
+				goto fail;
+
 			ret = gnutls_pubkey_import_url(pubkey, objurl, flags);
 			if (ret < 0)
 				goto fail;
@@ -169,7 +182,10 @@ const char *get_key_algo_type(gnutls_pkcs11_obj_type_t otype, const char *objurl
 			gnutls_pubkey_deinit(pubkey);
 			return p;
 		case GNUTLS_PKCS11_OBJ_PRIVKEY:
-			gnutls_privkey_init(&privkey);
+			ret = gnutls_privkey_init(&privkey);
+			if (ret < 0)
+				goto fail;
+
 			ret = gnutls_privkey_import_url(privkey, objurl, flags);
 			if (ret < 0)
 				goto fail;
@@ -217,12 +233,17 @@ pkcs11_list(FILE * outfile, const char *url, int type, unsigned int flags,
 	gnutls_x509_ext_st *exts;
 	unsigned exts_size;
 	unsigned int obj_flags = flags;
+	time_t exp;
 
 	pkcs11_common(info);
 
 	FIX(url, outfile, detailed, info);
 
-	gnutls_pkcs11_token_get_flags(url, &flags);
+	ret = gnutls_pkcs11_token_get_flags(url, &flags);
+	if (ret < 0) {
+		flags = 0;
+	}
+
 	if (flags & GNUTLS_PKCS11_TOKEN_TRUSTED)
 		print_exts = 1;
 
@@ -256,7 +277,7 @@ pkcs11_list(FILE * outfile, const char *url, int type, unsigned int flags,
 	}
 
 	for (i = 0; i < crt_list_size; i++) {
-		char buf[128];
+		char buf[256];
 		size_t size;
 		const char *p;
 		unsigned int oflags;
@@ -283,7 +304,7 @@ pkcs11_list(FILE * outfile, const char *url, int type, unsigned int flags,
 		if (otype == GNUTLS_PKCS11_OBJ_PRIVKEY ||
 		    otype == GNUTLS_PKCS11_OBJ_PUBKEY ||
 		    otype == GNUTLS_PKCS11_OBJ_X509_CRT) {
-			p = get_key_algo_type(otype, output, obj_flags);
+			p = get_key_algo_type(otype, output, obj_flags, &exp);
 		}
 
 		if (p) {
@@ -294,6 +315,13 @@ pkcs11_list(FILE * outfile, const char *url, int type, unsigned int flags,
 				gnutls_pkcs11_type_get_name(otype));
 		}
 
+		if (otype == GNUTLS_PKCS11_OBJ_X509_CRT && exp != -1) {
+			if (exp == GNUTLS_X509_NO_WELL_DEFINED_EXPIRATION)
+				fprintf(outfile, "\tExpires: Never\n");
+			else
+				fprintf(outfile, "\tExpires: %s", ctime(&exp));
+		}
+
 		gnutls_free(output);
 
 		size = sizeof(buf);
@@ -327,11 +355,16 @@ pkcs11_list(FILE * outfile, const char *url, int type, unsigned int flags,
 					       GNUTLS_PKCS11_OBJ_ID_HEX,
 					       buf, &size);
 		if (ret < 0) {
-			fprintf(stderr, "Error in %s:%d: %s\n", __func__,
-				__LINE__, gnutls_strerror(ret));
-			app_exit(1);
+			if (ret == GNUTLS_E_SHORT_MEMORY_BUFFER) {
+				fprintf(outfile, "\tID: (too long)\n");
+			} else {
+				fprintf(stderr, "Error in %s:%d: %s\n", __func__,
+					__LINE__, gnutls_strerror(ret));
+				app_exit(1);
+			}
+		} else {
+			fprintf(outfile, "\tID: %s\n", buf);
 		}
-		fprintf(outfile, "\tID: %s\n", buf);
 
 		if (otype == GNUTLS_PKCS11_OBJ_X509_CRT && print_exts > 0) {
 			ret = gnutls_pkcs11_obj_get_exts(crt_list[i], &exts, &exts_size, 0);
