@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2004-2012 Free Software Foundation, Inc.
  * Copyright (C) 2001,2002 Paul Sheer
- * Copyright (C) 2016-2017 Red Hat, Inc.
+ * Copyright (C) 2016-2018 Red Hat, Inc.
  * Portions Copyright (C) 2002,2003 Nikos Mavrogiannopoulos
  *
  * This file is part of GnuTLS.
@@ -138,7 +138,7 @@ LIST_TYPE_DECLARE(listener_item, char *http_request; char *http_response;
 		  int listen_socket; int fd;
 		  gnutls_session_t tls_session;
 		  int handshake_ok;
-		  int no_close;
+		  int close_ok;
 		  time_t start;
     );
 
@@ -156,7 +156,7 @@ static void listener_free(listener_item * j)
 	free(j->http_request);
 	free(j->http_response);
 	if (j->fd >= 0) {
-		if (j->no_close == 0)
+		if (j->close_ok)
 			gnutls_bye(j->tls_session, GNUTLS_SHUT_WR);
 		shutdown(j->fd, 2);
 		close(j->fd);
@@ -494,10 +494,11 @@ static char *peer_print_info(gnutls_session_t session, int *ret_length,
 	const char *tmp;
 	unsigned char sesid[32];
 	size_t i, sesid_size;
-	char *http_buffer;
+	char *http_buffer, *desc;
 	gnutls_kx_algorithm_t kx_alg;
 	size_t len = 20 * 1024 + strlen(header);
 	char *crtinfo = NULL, *crtinfo_old = NULL;
+	gnutls_protocol_t version;
 	size_t ncrtinfo = 0;
 
 	if (verbose == 0) {
@@ -566,6 +567,8 @@ static char *peer_print_info(gnutls_session_t session, int *ret_length,
 
 	strcpy(http_buffer, HTTP_BEGIN);
 
+	version = gnutls_protocol_get_version(session);
+
 	/* print session_id */
 	sesid_size = sizeof(sesid);
 	gnutls_session_get_id(session, sesid, &sesid_size);
@@ -574,7 +577,7 @@ static char *peer_print_info(gnutls_session_t session, int *ret_length,
 		snprintf(tmp_buffer, tmp_buffer_size, "%.2X", sesid[i]);
 	snprintf(tmp_buffer, tmp_buffer_size, "</i></p>\n");
 	snprintf(tmp_buffer, tmp_buffer_size,
-		 "<h5>If your browser supports session resuming, then you should see the "
+		 "<h5>If your browser supports session resumption, then you should see the "
 		 "same session ID, when you press the <b>reload</b> button.</h5>\n");
 
 	/* Here unlike print_info() we use the kx algorithm to distinguish
@@ -605,39 +608,35 @@ static char *peer_print_info(gnutls_session_t session, int *ret_length,
 #endif
 
 #ifdef ENABLE_PSK
-	if (kx_alg == GNUTLS_KX_PSK) {
+	if (kx_alg == GNUTLS_KX_PSK && gnutls_psk_server_get_username(session)) {
 		snprintf(tmp_buffer, tmp_buffer_size,
 			 "<p>Connected as user '%s'.</p>\n",
 			 gnutls_psk_server_get_username(session));
 	}
 #endif
 
-#ifdef ENABLE_ANON
-	if (kx_alg == GNUTLS_KX_ANON_DH) {
-		snprintf(tmp_buffer, tmp_buffer_size,
-			 "<p> Connect using anonymous DH (prime of %d bits)</p>\n",
-			 gnutls_dh_get_prime_bits(session));
-	}
-#endif
-
-	if (kx_alg == GNUTLS_KX_DHE_RSA || kx_alg == GNUTLS_KX_DHE_DSS) {
-		snprintf(tmp_buffer, tmp_buffer_size,
-			 "Ephemeral DH using prime of <b>%d</b> bits.<br>\n",
-			 gnutls_dh_get_prime_bits(session));
-	}
 
 	/* print session information */
 	strcat(http_buffer, "<P>\n");
 
 	tmp =
-	    gnutls_protocol_get_name(gnutls_protocol_get_version(session));
+	    gnutls_protocol_get_name(version);
 	if (tmp == NULL)
 		tmp = str_unknown;
 	snprintf(tmp_buffer, tmp_buffer_size,
 		 "<TABLE border=1><TR><TD>Protocol version:</TD><TD>%s</TD></TR>\n",
 		 tmp);
 
-	if (gnutls_auth_get_type(session) == GNUTLS_CRD_CERTIFICATE) {
+	desc = gnutls_session_get_desc(session);
+	if (desc) {
+		snprintf(tmp_buffer, tmp_buffer_size,
+			"<TR><TD>Description:</TD><TD>%s</TD></TR>\n",
+			desc);
+		gnutls_free(desc);
+	}
+
+	if (gnutls_auth_get_type(session) == GNUTLS_CRD_CERTIFICATE &&
+	    gnutls_certificate_type_get(session) != GNUTLS_CRT_X509) {
 		tmp =
 		    gnutls_certificate_type_get_name
 		    (gnutls_certificate_type_get(session));
@@ -648,17 +647,42 @@ static char *peer_print_info(gnutls_session_t session, int *ret_length,
 			 tmp);
 	}
 
-	tmp = gnutls_kx_get_name(kx_alg);
-	if (tmp == NULL)
-		tmp = str_unknown;
-	snprintf(tmp_buffer, tmp_buffer_size,
-		 "<TR><TD>Key Exchange:</TD><TD>%s</TD></TR>\n", tmp);
+	if (version < GNUTLS_TLS1_3) {
+		tmp = gnutls_kx_get_name(kx_alg);
+		if (tmp == NULL)
+			tmp = str_unknown;
+		snprintf(tmp_buffer, tmp_buffer_size,
+			 "<TR><TD>Key Exchange:</TD><TD>%s</TD></TR>\n", tmp);
 
-	tmp = gnutls_compression_get_name(gnutls_compression_get(session));
-	if (tmp == NULL)
-		tmp = str_unknown;
-	snprintf(tmp_buffer, tmp_buffer_size,
-		 "<TR><TD>Compression</TD><TD>%s</TD></TR>\n", tmp);
+#ifdef ENABLE_ANON
+		if (kx_alg == GNUTLS_KX_ANON_DH) {
+			snprintf(tmp_buffer, tmp_buffer_size,
+				 "<p> Connect using anonymous DH (prime of %d bits)</p>\n",
+				 gnutls_dh_get_prime_bits(session));
+		}
+#endif
+
+		if (kx_alg == GNUTLS_KX_DHE_RSA || kx_alg == GNUTLS_KX_DHE_DSS) {
+			snprintf(tmp_buffer, tmp_buffer_size,
+				 "Ephemeral DH using prime of <b>%d</b> bits.<br>\n",
+				 gnutls_dh_get_prime_bits(session));
+		}
+
+		tmp = gnutls_compression_get_name(gnutls_compression_get(session));
+		if (tmp == NULL)
+			tmp = str_unknown;
+		snprintf(tmp_buffer, tmp_buffer_size,
+			 "<TR><TD>Compression</TD><TD>%s</TD></TR>\n", tmp);
+
+		tmp = gnutls_cipher_suite_get_name(kx_alg,
+						   gnutls_cipher_get(session),
+						   gnutls_mac_get(session));
+		if (tmp == NULL)
+			tmp = str_unknown;
+		snprintf(tmp_buffer, tmp_buffer_size,
+			 "<TR><TD>Ciphersuite</TD><TD>%s</TD></TR></p></TABLE>\n",
+			 tmp);
+	}
 
 	tmp = gnutls_cipher_get_name(gnutls_cipher_get(session));
 	if (tmp == NULL)
@@ -672,14 +696,6 @@ static char *peer_print_info(gnutls_session_t session, int *ret_length,
 	snprintf(tmp_buffer, tmp_buffer_size,
 		 "<TR><TD>MAC</TD><TD>%s</TD></TR>\n", tmp);
 
-	tmp = gnutls_cipher_suite_get_name(kx_alg,
-					   gnutls_cipher_get(session),
-					   gnutls_mac_get(session));
-	if (tmp == NULL)
-		tmp = str_unknown;
-	snprintf(tmp_buffer, tmp_buffer_size,
-		 "<TR><TD>Ciphersuite</TD><TD>%s</TD></TR></p></TABLE>\n",
-		 tmp);
 
 	if (crtinfo) {
 		snprintf(tmp_buffer, tmp_buffer_size,
@@ -1263,7 +1279,7 @@ static void retry_handshake(listener_item *j)
 		do {
 			ret = gnutls_alert_send_appropriate(j->tls_session, r);
 		} while (ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED);
-		j->no_close = 1;
+		j->close_ok = 0;
 	} else if (r == 0) {
 		if (gnutls_session_is_resumed(j->tls_session) != 0 && verbose != 0)
 			printf("*** This is a resumed session\n");
@@ -1281,14 +1297,16 @@ static void retry_handshake(listener_item *j)
 			print_info(j->tls_session, verbose, verbose);
 		}
 
+		j->close_ok = 1;
 		j->handshake_ok = 1;
 	}
 }
 
 static void try_rehandshake(listener_item *j)
 {
-int r, ret;
+	int r, ret;
 	fprintf(stderr, "*** Received hello message\n");
+
 	do {
 		r = gnutls_handshake(j->tls_session);
 	} while (r == GNUTLS_E_INTERRUPTED || r == GNUTLS_E_AGAIN);
@@ -1300,6 +1318,7 @@ int r, ret;
 		fprintf(stderr, "Error in rehandshake: %s\n", gnutls_strerror(r));
 		j->http_state = HTTP_STATE_CLOSING;
 	} else {
+		j->close_ok = 1;
 		j->http_state = HTTP_STATE_REQUEST;
 	}
 }
@@ -1407,7 +1426,7 @@ static void tcp_server(const char *name, int port)
 					    (j->tls_session, accept_fd);
 					set_read_funcs(j->tls_session);
 					j->handshake_ok = 0;
-					j->no_close = 0;
+					j->close_ok = 0;
 
 					if (verbose != 0) {
 						ctt = ctime(&tt);
@@ -1461,7 +1480,7 @@ static void tcp_server(const char *name, int port)
 									ret = gnutls_alert_send_appropriate(j->tls_session, r);
 								} while (ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED);
 								GERR(r);
-								j->no_close = 1;
+								j->close_ok = 0;
 							}
 						}
 					} else {
