@@ -25,6 +25,9 @@ typedef struct {
 	struct sockaddr_storage connect_addr;
 	socklen_t connect_addrlen;
 
+	FILE *server_trace;
+	FILE *client_trace;
+
 	/* resumption data */
 	gnutls_datum_t rdata;
 } socket_st;
@@ -42,8 +45,12 @@ ssize_t socket_send(const socket_st * socket, const void *buffer,
 ssize_t socket_send_range(const socket_st * socket, const void *buffer,
 			  int buffer_size, gnutls_range_st * range);
 void
-socket_open(socket_st * hd, const char *hostname, const char *service,
-	    const char *app_proto, int flags, const char *msg, gnutls_datum_t *rdata);
+socket_open2(socket_st * hd, const char *hostname, const char *service,
+	    const char *app_proto, int flags, const char *msg, gnutls_datum_t *rdata,
+	    FILE *server_trace, FILE *client_trace);
+
+#define socket_open(hd, host, service, app_proto, flags, msg, rdata) \
+	socket_open2(hd, host, service, app_proto, flags, msg, rdata, NULL, NULL)
 
 void socket_bye(socket_st * socket, unsigned polite);
 
