@@ -47,6 +47,7 @@ int main()
 #include <gnutls/abstract.h>
 #include <gnutls/urls.h>
 #include <signal.h>
+#include <assert.h>
 
 #include "cert-common.h"
 #include "utils.h"
@@ -93,7 +94,7 @@ static void client(int fd)
 	gnutls_init(&session, GNUTLS_CLIENT);
 
 	/* Use default priorities */
-	gnutls_priority_set_direct(session, "NORMAL", NULL);
+	assert(gnutls_priority_set_direct(session, "NORMAL:-VERS-ALL:+VERS-TLS1.2", NULL)>=0);
 
 	gnutls_credentials_set(session, GNUTLS_CRD_CERTIFICATE, x509_cred);
 
@@ -150,7 +151,7 @@ static void server(int fd)
 	 */
 	gnutls_certificate_allocate_credentials(&x509_cred);
 	ret = gnutls_certificate_set_x509_key_file(x509_cred, "nomyurl:cert", "nomyurl:key",
-					     	  GNUTLS_X509_FMT_PEM);
+						   GNUTLS_X509_FMT_PEM);
 	if (ret != GNUTLS_E_FILE_ERROR) {
 		fail("server: gnutls_certificate_set_x509_key_file unexpected error (%s)\n\n",
 		     gnutls_strerror(ret));
@@ -158,7 +159,7 @@ static void server(int fd)
 	}
 
 	ret = gnutls_certificate_set_x509_key_file(x509_cred, "myurl:cert", "myurl:key",
-					     	  GNUTLS_X509_FMT_PEM);
+						   GNUTLS_X509_FMT_PEM);
 	if (ret < 0) {
 		fail("server: gnutls_certificate_set_x509_key_file (%s)\n\n",
 		     gnutls_strerror(ret));
@@ -170,7 +171,7 @@ static void server(int fd)
 	/* avoid calling all the priority functions, since the defaults
 	 * are adequate.
 	 */
-	gnutls_priority_set_direct(session, "NORMAL", NULL);
+	assert(gnutls_priority_set_direct(session, "NORMAL:-VERS-ALL:+VERS-TLS1.2", NULL)>=0);
 
 	gnutls_credentials_set(session, GNUTLS_CRD_CERTIFICATE, x509_cred);
 
