@@ -59,7 +59,7 @@ static void try(const char *client_prio, int expected)
 	int cret = GNUTLS_E_AGAIN;
 	unsigned flags = 0;
 	unsigned dtls = 0;
-	const char *server_prio = "NORMAL";
+	const char *server_prio = "NORMAL:+VERS-TLS-ALL";
 
 	if (expected >= GNUTLS_DTLS_VERSION_MIN && expected <= GNUTLS_DTLS_VERSION_MAX) {
 		dtls = 1;
@@ -84,7 +84,7 @@ static void try(const char *client_prio, int expected)
 
 	gnutls_init(&server, GNUTLS_SERVER|flags);
 	gnutls_credentials_set(server, GNUTLS_CRD_CERTIFICATE,
-			       serverx509cred);
+				serverx509cred);
 
 	assert(gnutls_priority_set_direct(server,
 				   server_prio,
@@ -109,7 +109,7 @@ static void try(const char *client_prio, int expected)
 		exit(1);
 
 	ret = gnutls_credentials_set(client, GNUTLS_CRD_CERTIFICATE,
-			       clientx509cred);
+				clientx509cred);
 	if (ret < 0)
 		exit(1);
 
@@ -162,6 +162,8 @@ void doit(void)
 	reset_buffers();
 	try("NORMAL:-VERS-TLS-ALL:+VERS-TLS1.2", GNUTLS_TLS1_2);
 	reset_buffers();
+	try("NORMAL:-VERS-TLS-ALL:+VERS-TLS1.3", GNUTLS_TLS1_3);
+	reset_buffers();
 #ifdef ENABLE_SSL3
 	try("NORMAL:-VERS-TLS-ALL:+VERS-SSL3.0", -1);
 	reset_buffers();
